// $URL: http://subversion:8080/svn/gsc/trunk/drivers/gsc_common/api/intime/os_mbox.c $
// $Rev: 43206 $
// $Date: 2018-07-08 17:03:14 -0500 (Sun, 08 Jul 2018) $

// INtime: API Library: source file

#include "main.h"



//*****************************************************************************
// Return Values:
//	0  = success
//	<0 = failure, value is -errno or -INtime error code
//	>0 = never returned
int os_mbox_alloc(os_mbox_t** mbox)
{
	os_mbox_t*	ptr;
	int			ret;

	for (;;)	// A convenience loop.
	{
		if (mbox == NULL)
		{
			ret	= -EINVAL;
			break;
		}

		ptr	= malloc(sizeof(os_mbox_t));

		if (ptr == NULL)
		{
			mbox[0]	= NULL;
			ret		= -ENOMEM;
			break;
		}

		memset(ptr, 0, sizeof(os_mbox_t));
		ptr->mbox	= CreateRtMailbox(DATA_MAILBOX);

		if (ptr->mbox == BAD_RTHANDLE)
		{
			free(ptr);
			mbox[0]	= NULL;
			ret		= os_get_error_status();
			break;
		}

		mbox[0]	= ptr;
		ret		= 0;
		break;
	}

	return(ret);
}



//*****************************************************************************
// Return Values:
//	0  = success
//	<0 = failure, value is -errno or -INtime error code
//	>0 = never returned
int os_mbox_free(os_mbox_t** mbox)
{
	int	ret;

	for (;;)	// A convenience loop.
	{
		if (mbox == NULL)
		{
			// There is nothing to do.
			ret	= 0;
			break;
		}

		if (mbox[0] == NULL)
		{
			// There is nothing to do.
			ret	= 0;
			break;
		}

		if (mbox[0]->mbox != BAD_RTHANDLE)
			DeleteRtMailbox(mbox[0]->mbox);

		memset(mbox[0], 0, sizeof(os_mbox_t));
		free(mbox[0]);
		mbox[0]	= NULL;
		ret		= 0;
		break;
	}

	return(ret);
}


