// $URL: http://subversion:8080/svn/gsc/trunk/drivers/gsc_common/api/intime/os_write.c $
// $Rev: 43206 $
// $Date: 2018-07-08 17:03:14 -0500 (Sun, 08 Jul 2018) $

// INtime: API Library: source file

#include "main.h"



//*****************************************************************************
// Return Values:
//	>=0  = success, number of bytes transferred
//	<0 = failure, value is -errno or -INtime error code
int os_api_write(int fd, const void* src, size_t bytes)
{
	os_file_t*	fp		= NULL;
	os_mbox_t*	mbox	= NULL;
	RTHANDLE	mem		= BAD_RTHANDLE;
	os_msg_t*	msg		= NULL;
	u32			offset	= 0;
	int			ret	= 0;
	WORD		size;
	BOOLEAN		sts;

	for (;;)	// a convenience loop
	{
		if (os_api_global.initialized == 0)
		{
			// The library has not been initialized.
			ret	= -EPROTO;
			break;
		}

		// Acquire file access.
		ret	= os_file_lock(fd, &fp);

		if (ret)
		{
			// There was a problem!
			break;
		}

		// Perform basic validation.

		if (bytes == 0)
		{
			// There is nothing to do.
			ret	= 0;
			break;
		}

		if (src == NULL)
		{
			ret	= -EINVAL;
			break;
		}

		// Begin the WRITE operation.

		// Acquire a mailbox for the receive message.
		ret	= os_mbox_alloc(&mbox);

		if (ret)
			break;

		// Allocate a message structure.
		ret	= os_msg_alloc(&msg);

		if (ret)
			break;

		// Create a memory handle for the buffer.
		ret	= os_mem_handle_create((void*) src, bytes, &offset, &mem);

		if (ret)
			break;

		// Initialize and send the message.
		memset(msg, 0, sizeof(os_msg_t));
		msg->status			= OS_MSG_STS_OK;
		msg->msg_type		= OS_MSG_TYPE_WRITE;
		msg->size			= OS_MSG_SIZE_WRITE;
		msg->mbox			= mbox->mbox;
		msg->u.write.count	= bytes;
		msg->u.write.mem	= (u32) mem;
		msg->u.write.offset	= offset;
		sts					= SendRtData(fp->dev_mbox, msg, OS_MSG_SIZE_WRITE);

		if (sts == FALSE)
		{
			ret	= os_get_error_status();
			break;
		}

		// Release the file pointer and don't use it after this point.
		os_file_unlock(&fp);

		// Get and process the response.
		size	= ReceiveRtData(mbox->mbox, msg, WAIT_FOREVER);

		if (size == 0)
		{
			// The operation failed.
			ret	= os_get_error_status();
			break;
		}

		if ((size != OS_MSG_SIZE_WRITE) || (size != msg->size))
		{
			ret	= -EIO;
			break;
		}

		switch (msg->status)
		{
			default:
			case OS_MSG_STS_MSG_TYPE:

				ret	= -EBADMSG;
				break;

			case OS_MSG_STS_OK:
			case OS_MSG_STS_MSG_SIZE:
			case OS_MSG_STS_MEM_MAP:

				ret	= msg->u.write.ret;
				break;

			case OS_MSG_STS_THREAD:

				ret	= -ENOEXEC;
				break;
		}

		break;
	}

	os_mem_handle_destroy(&mem);
	os_msg_free(&msg);
	os_mbox_free(&mbox);
	os_file_unlock(&fp);
	return(ret);
}


