// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AIO/16AIO_Linux_4.x.x.x_DN/driver/io.c $
// $Rev: 43208 $
// $Date: 2018-07-08 17:20:44 -0500 (Sun, 08 Jul 2018) $

// 16AIO: Device Driver: source file

#include "main.h"



/******************************************************************************
*
*	Function:	dev_io_close
*
*	Purpose:
*
*		Cleanup the I/O stuff for the device as it is being closed.
*
*	Arguments:
*
*		dev	The data for the device of interest.
*
*	Returned:
*
*		None.
*
******************************************************************************/

void dev_io_close(dev_data_t* dev)
{
	dev->rx.dma_channel	= NULL;
	dev->tx.dma_channel	= NULL;
}



/******************************************************************************
*
*	Function:	dev_io_create
*
*	Purpose:
*
*		Perform I/O based initialization as the driver is being loaded.
*
*	Arguments:
*
*		dev		The data for the device of interest.
*
*	Returned:
*
*		0		All went well.
*		< 0		The error status for the problem encounterred.
*
******************************************************************************/

int dev_io_create(dev_data_t* dev)
{
	int	ret;

	dev->rx.bytes_per_sample	= 4;
	dev->rx.io_reg_offset		= GSC_REG_OFFSET(AIO_GSC_IDBR);
	dev->rx.io_reg_vaddr		= dev->vaddr.gsc_idbr_32;

	dev->tx.bytes_per_sample	= 4;
	dev->tx.io_reg_offset		= GSC_REG_OFFSET(AIO_GSC_ODBR);
	dev->tx.io_reg_vaddr		= dev->vaddr.gsc_odbr_32;

	ret	= gsc_io_create(dev, &dev->rx, 32L * 1024 * 4);

	if (ret == 0)
		ret	= gsc_io_create(dev, &dev->tx, 32L * 1024 * 4);

	return(ret);
}



/******************************************************************************
*
*	Function:	dev_io_destroy
*
*	Purpose:
*
*		Perform I/O based cleanup as the driver is being unloaded.
*
*	Arguments:
*
*		dev		The data for the device of interest.
*
*	Returned:
*
*		None.
*
******************************************************************************/

void dev_io_destroy(dev_data_t* dev)
{
	dev_io_close(dev);	// Just in case.
	gsc_io_destroy(dev, &dev->rx);
	gsc_io_destroy(dev, &dev->tx);
}



/******************************************************************************
*
*	Function:	dev_io_open
*
*	Purpose:
*
*		Perform I/O based initialization as the device is being opened.
*
*	Arguments:
*
*		dev		The data for the device of interest.
*
*	Returned:
*
*		0		All went well.
*		< 0		The code for the error seen.
*
******************************************************************************/

int dev_io_open(dev_data_t* dev)
{
	dev_io_close(dev);	// Just in case.
	return(0);
}


