// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AIO/16AIO_Linux_4.x.x.x_DN/driver/ioctl.c $
// $Rev: 43208 $
// $Date: 2018-07-08 17:20:44 -0500 (Sun, 08 Jul 2018) $

// 16AIO: Device Driver: source file

#include "main.h"



// #defines *******************************************************************

#define BCR_AUTO_CAL_START		D13
#define BCR_AUTO_CAL_PASS		D14
#define INIT_START				D15

#define ICR_IRQ0_AUTO_CAL_DONE	0x0001
#define ICR_IRQ0_MASK			0x0007
#define ICR_IRQ0_ACTIVE			0x0008
#define ICR_IRQ1_ACTIVE			0x0080
#define ICR_IRQ2_ACTIVE			0x0800

#define SSCR_SCAN_1				D11
#define SSCR_SCAN_2				D17
#define SSCR_SCAN_MULTI			(D0 | D1)



//*****************************************************************************
static void _nrate_range(dev_data_t* dev, int* max, int* min)
{
	if (dev->cache.nrate_ai_min < dev->cache.nrate_ao_min)
		min[0]	= dev->cache.nrate_ai_min;
	else
		min[0]	= dev->cache.nrate_ao_min;

	if (dev->cache.nrate_ai_max < dev->cache.nrate_ao_max)
		max[0]	= dev->cache.nrate_ai_max;
	else
		max[0]	= dev->cache.nrate_ao_max;
}



//*****************************************************************************
static int _query(dev_data_t* dev, void* arg)
{
	int		ret	= 0;
	s32*	ptr	= (s32*) arg;

	switch (ptr[0])
	{
		default:	ptr[0]	= AIO_IOCTL_QUERY_ERROR;
					ret		= -EINVAL;
					break;

		case AIO_QUERY_AUTO_CAL_MS:		ptr[0]	= dev->cache.auto_cal_ms;	break;
		case AIO_QUERY_COUNT:			ptr[0]	= AIO_QUERY_COUNT + 1;		break;
		case AIO_QUERY_DEVICE_TYPE:		ptr[0]	= dev->board_type;			break;
		case AIO_QUERY_FIFO_SIZE_RX:	ptr[0]	= dev->cache.fifo_size_rx;	break;
		case AIO_QUERY_FIFO_SIZE_TX:	ptr[0]	= dev->cache.fifo_size_tx;	break;
		case AIO_QUERY_FGEN_AI_MAX:		ptr[0]	= dev->cache.fgen_ai_max;	break;
		case AIO_QUERY_FGEN_AI_MIN:		ptr[0]	= dev->cache.fgen_ai_min;	break;
		case AIO_QUERY_FGEN_AO_MAX:		ptr[0]	= dev->cache.fgen_ao_max;	break;
		case AIO_QUERY_FGEN_AO_MIN:		ptr[0]	= dev->cache.fgen_ao_min;	break;
		case AIO_QUERY_FREF_DEFAULT:	ptr[0]	= dev->cache.fref_default;	break;
		case AIO_QUERY_FSAMP_AI_MAX:	ptr[0]	= dev->cache.fsamp_ai_max;	break;
		case AIO_QUERY_FSAMP_AI_MIN:	ptr[0]	= dev->cache.fsamp_ai_min;	break;
		case AIO_QUERY_FSAMP_AO_MAX:	ptr[0]	= dev->cache.fsamp_ao_max;	break;
		case AIO_QUERY_FSAMP_AO_MIN:	ptr[0]	= dev->cache.fsamp_ao_min;	break;
		case AIO_QUERY_INIT_MS:			ptr[0]	= dev->cache.initialize_ms;	break;
		case AIO_QUERY_NRATE_AI_MASK:	ptr[0]	= dev->cache.nrate_ai_mask;	break;
		case AIO_QUERY_NRATE_AI_MAX:	ptr[0]	= dev->cache.nrate_ai_max;	break;
		case AIO_QUERY_NRATE_AI_MIN:	ptr[0]	= dev->cache.nrate_ai_min;	break;
		case AIO_QUERY_NRATE_AO_MASK:	ptr[0]	= dev->cache.nrate_ao_mask;	break;
		case AIO_QUERY_NRATE_AO_MAX:	ptr[0]	= dev->cache.nrate_ao_max;	break;
		case AIO_QUERY_NRATE_AO_MIN:	ptr[0]	= dev->cache.nrate_ao_min;	break;
		case AIO_QUERY_RES_BITS:		ptr[0]	= dev->cache.res_bits;		break;
	}

	return(ret);
}



//*****************************************************************************
int initialize_ioctl(dev_data_t* dev, void* arg)
{
	long	ms;
	u32		reg;
	int		ret;

	// Manually wait for completion to insure that initialization completes
	// even if we get a signal, such as ^c.
	os_reg_mem_tx_u32(dev, dev->vaddr.gsc_bcr_32, INIT_START);
	ms	= dev->cache.initialize_ms + 5000;
	ret	= gsc_poll_u32(dev, ms, dev->vaddr.gsc_bcr_32, INIT_START, 0);
	reg	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_bcr_32);

	if (ret)
	{
		printf(	"%s: INITIALIZATION TIMED OUT (%ld ms)\n",
				dev->model,
				(long) ms);
	}

	if (reg & INIT_START)
	{
		ret	= -EIO;
		printf(	"%s: INITIALIZATION DID NOT COMPLETE (%ld ms)\n",
				dev->model,
				(long) ms);
	}

	os_time_sleep_ms(10);	// Initialize post delay.

	// Initialize the software settings.
	dev->rx.bytes_per_sample	= 4;
	dev->rx.io_mode				= AIO_IO_MODE_DEFAULT;
	dev->rx.pio_threshold		= 32;
	dev->rx.timeout_s			= AIO_IO_TIMEOUT_DEFAULT;

	dev->tx.bytes_per_sample	= 4;
	dev->tx.io_mode				= AIO_IO_MODE_DEFAULT;
	dev->tx.pio_threshold		= 32;
	dev->tx.timeout_s			= AIO_IO_TIMEOUT_DEFAULT;
	return(ret);
}



//*****************************************************************************
static int _auto_cal_start(dev_data_t* dev, void* arg)
{
	// Initiate auto-calibration.
	os_reg_mem_mx_u32(NULL, dev->vaddr.gsc_bcr_32, BCR_AUTO_CAL_START, BCR_AUTO_CAL_START);
	return(0);
}



//*****************************************************************************
static int _auto_calibrate(dev_data_t* dev, void* arg)
{
	u32			mask;
	long		ms;
	u32			reg;
	int			ret;
	os_sem_t	sem;
	int			sts;
	u32			value;
	gsc_wait_t	wt;

	// Safely select the Auto-Cal Done interrupt.

	if (dev->cache.icr_irq_bug)
		value	= ICR_IRQ0_AUTO_CAL_DONE;
	else
		value	= ICR_IRQ0_AUTO_CAL_DONE | ICR_IRQ1_ACTIVE | ICR_IRQ2_ACTIVE;

	mask	= ICR_IRQ0_ACTIVE | ICR_IRQ1_ACTIVE | ICR_IRQ2_ACTIVE | ICR_IRQ0_MASK;
	os_reg_mem_mx_u32(dev, dev->vaddr.gsc_icr_32, value, mask);

	// Wait for the local interrupt.
	os_sem_create(&sem);	// dummy, required for wait operations.
	ms				= dev->cache.auto_cal_ms + 5000;
	memset(&wt, 0, sizeof(wt));
	wt.flags		= GSC_WAIT_FLAG_INTERNAL;
	wt.gsc			= AIO_WAIT_GSC_AUTO_CAL_DONE;
	wt.timeout_ms	= ms;

	ret				= gsc_wait_event(dev, &wt, _auto_cal_start, NULL, &sem);
	os_sem_destroy(&sem);

	if (wt.flags & GSC_WAIT_FLAG_TIMEOUT)
	{
		ret	= ret ? ret : -ETIMEDOUT;
		printf(	"%s: AUTO-CALIBRATION TIMED OUT (%ld ms).\n",
				dev->model,
				(long) ms);
	}

	// Wait for the bit to clear
	sts	= gsc_poll_u32(dev, ms, dev->vaddr.gsc_bcr_32, BCR_AUTO_CAL_START, 0);

	if (sts)
	{
		ret	= ret ? ret : -EIO;
		printf(	"%s: AUTO-CALIBRATION DID NOT COMPLETE (%ld ms)\n",
				dev->model,
				(long) ms * 2);
	}

	// Check the completion status.
	reg	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_bcr_32);

	if ((reg & BCR_AUTO_CAL_PASS) == 0)
	{
		ret	= ret ? ret : -EIO;
		printf(	"%s: AUTO-CALIBRATION FAILED (%ld ms)\n",
				dev->model,
				(long) ms);
	}

	return(ret);
}



//*****************************************************************************
static int _auto_cal_sts(dev_data_t* dev, void* arg)
{
	s32*	parm	= (s32*) arg;
	u32		reg;
	s32		value;

	reg	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_bcr_32);

	if (reg & BCR_AUTO_CAL_START)
		value	= AIO_AUTO_CAL_STS_ACTIVE;
	else if (reg & BCR_AUTO_CAL_PASS)
		value	= AIO_AUTO_CAL_STS_PASS;
	else
		value	= AIO_AUTO_CAL_STS_FAIL;

	parm[0]	= value;
	return(0);
}



//*****************************************************************************
static int _ai_buf_clear(dev_data_t* dev, void* arg)
{
	u32		mask	= D15;
	long	ms		= (dev->rx.timeout_s ? dev->rx.timeout_s : 1) * 1000;
	int		ret;

	// Clear the buffer.
	os_reg_mem_mx_u32(dev, dev->vaddr.gsc_ibcr_32, mask, mask);

	// Wait for the bit to clear.
	ret	= gsc_poll_u32(dev, ms, dev->vaddr.gsc_ibcr_32, mask, 0);
	return(ret);
}



//*****************************************************************************
static int _ai_buf_thr_lvl(dev_data_t* dev, void* arg)
{
	int	ret;

	ret	= gsc_s32_range_reg(dev, arg, 0, 0x7FFF, dev->vaddr.gsc_ibcr_32, 14, 0);
	return(ret);
}



//*****************************************************************************
static int _ai_buf_thr_sts(dev_data_t* dev, void* arg)
{
	static const s32	options[]	=
	{
		AIO_AI_BUF_THR_STS_CLEAR,
		AIO_AI_BUF_THR_STS_SET,
		-1	// terminate list
	};

	s32*	ptr	= (s32*) arg;
	int		ret;

	ptr[0]	= -1;
	ret		= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_ibcr_32, 16, 16);
	return(ret);
}



//*****************************************************************************
static int _ai_mode(dev_data_t* dev, void* arg)
{
	static const s32	options[]	=
	{
		AIO_AI_MODE_DIFF,
		AIO_AI_MODE_SINGLE,
		AIO_AI_MODE_ZERO,
		AIO_AI_MODE_VREF,
		AIO_AI_MODE_AO_0,
		AIO_AI_MODE_AO_1,
		AIO_AI_MODE_AO_2,
		AIO_AI_MODE_AO_3,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bcr_32, 2, 0);
	return(ret);
}



//*****************************************************************************
static int _ai_scan_1_chan(dev_data_t* dev, void* arg)
{
	int	ret;

	ret	= gsc_s32_range_reg(dev, arg, 0, 31, dev->vaddr.gsc_sscr_32, 16, 12);
	return(ret);
}



//*****************************************************************************
static int _ai_scan_clk_src(dev_data_t* dev, void* arg)
{
	static const s32	options[]	=
	{
		AIO_AI_SCAN_CLK_SRC_RGA,
		AIO_AI_SCAN_CLK_SRC_RGB,
		AIO_AI_SCAN_CLK_SRC_EXT,
		AIO_AI_SCAN_CLK_SRC_BCR,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_sscr_32, 3, 2);
	return(ret);
}



//*****************************************************************************
static int _ai_scan_size(dev_data_t* dev, void* arg)
{
	s32*	ptr		= (s32*) arg;
	int		ret		= 0;
	u32		sscr	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_sscr_32);

	if (ptr[0] == -1)
	{
		if (sscr & SSCR_SCAN_2)
		{
			ptr[0]	= AIO_AI_SCAN_SIZE_0_1;
		}
		else if (sscr & SSCR_SCAN_1)
		{
			ptr[0]	= AIO_AI_SCAN_SIZE_SINGLE;
		}
		else
		{
			switch (sscr & 0x3)
			{
				default:	ret		= -EIO;						break;
				case 0x0:	ptr[0]	= AIO_AI_SCAN_SIZE_0_3;		break;
				case 0x1:	ptr[0]	= AIO_AI_SCAN_SIZE_0_7;		break;
				case 0x2:	ptr[0]	= AIO_AI_SCAN_SIZE_0_15;	break;
				case 0x3:	ptr[0]	= AIO_AI_SCAN_SIZE_0_31;	break;
			}
		}
	}
	else
	{
		switch (ptr[0])
		{
			default:

				ret	= -EIO;
				break;

			case AIO_AI_SCAN_SIZE_SINGLE:

				os_reg_mem_mx_u32(dev, dev->vaddr.gsc_sscr_32, SSCR_SCAN_1, SSCR_SCAN_1);
				os_reg_mem_mx_u32(dev, dev->vaddr.gsc_sscr_32, 0, SSCR_SCAN_2);
				break;

			case AIO_AI_SCAN_SIZE_0_1:

				os_reg_mem_mx_u32(dev, dev->vaddr.gsc_sscr_32, 0, SSCR_SCAN_1);
				os_reg_mem_mx_u32(dev, dev->vaddr.gsc_sscr_32, SSCR_SCAN_2, SSCR_SCAN_2);
				break;

			case AIO_AI_SCAN_SIZE_0_3:

				os_reg_mem_mx_u32(dev, dev->vaddr.gsc_sscr_32, 0, SSCR_SCAN_1);
				os_reg_mem_mx_u32(dev, dev->vaddr.gsc_sscr_32, 0, SSCR_SCAN_2);
				os_reg_mem_mx_u32(dev, dev->vaddr.gsc_sscr_32, 0, SSCR_SCAN_MULTI);
				break;

			case AIO_AI_SCAN_SIZE_0_7:

				os_reg_mem_mx_u32(dev, dev->vaddr.gsc_sscr_32, 0, SSCR_SCAN_1);
				os_reg_mem_mx_u32(dev, dev->vaddr.gsc_sscr_32, 0, SSCR_SCAN_2);
				os_reg_mem_mx_u32(dev, dev->vaddr.gsc_sscr_32, 1, SSCR_SCAN_MULTI);
				break;

			case AIO_AI_SCAN_SIZE_0_15:

				os_reg_mem_mx_u32(dev, dev->vaddr.gsc_sscr_32, 0, SSCR_SCAN_1);
				os_reg_mem_mx_u32(dev, dev->vaddr.gsc_sscr_32, 0, SSCR_SCAN_2);
				os_reg_mem_mx_u32(dev, dev->vaddr.gsc_sscr_32, 2, SSCR_SCAN_MULTI);
				break;

			case AIO_AI_SCAN_SIZE_0_31:

				os_reg_mem_mx_u32(dev, dev->vaddr.gsc_sscr_32, 0, SSCR_SCAN_1);
				os_reg_mem_mx_u32(dev, dev->vaddr.gsc_sscr_32, 0, SSCR_SCAN_2);
				os_reg_mem_mx_u32(dev, dev->vaddr.gsc_sscr_32, 3, SSCR_SCAN_MULTI);
				break;
		}
	}

	return(ret);
}



//*****************************************************************************
static int _ai_sync(dev_data_t* dev, void* arg)
{
	os_reg_mem_mx_u32(dev, dev->vaddr.gsc_bcr_32, D12, D12);
	return(0);
}



//*****************************************************************************
static int _ao_buf_clear(dev_data_t* dev, void* arg)
{
	u32		mask	= D15;
	long	ms		= (dev->rx.timeout_s ? dev->tx.timeout_s : 1) * 1000;
	int		ret;

	// Clear the buffer.
	os_reg_mem_mx_u32(dev, dev->vaddr.gsc_obcr_32, mask, mask);

	// Wait for the bit to clear.
	ret	= gsc_poll_u32(dev, ms, dev->vaddr.gsc_obcr_32, mask, 0);
	return(ret);
}



//*****************************************************************************
static int _ao_buf_thr_lvl(dev_data_t* dev, void* arg)
{
	int	ret;

	ret	= gsc_s32_range_reg(dev, arg, 0, 0x7FFF, dev->vaddr.gsc_obcr_32, 14, 0);
	return(ret);
}



//*****************************************************************************
static int _ao_buf_thr_sts(dev_data_t* dev, void* arg)
{
	static const s32	options[]	=
	{
		AIO_AO_BUF_THR_STS_CLEAR,
		AIO_AO_BUF_THR_STS_SET,
		-1	// terminate list
	};

	s32*	ptr	= (s32*) arg;
	int		ret;

	ptr[0]	= -1;
	ret		= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_obcr_32, 16, 16);
	return(ret);
}



//*****************************************************************************
static int _ao_burst_clk_src(dev_data_t* dev, void* arg)
{
	static const s32	options[]	=
	{
		AIO_AO_BURST_CLK_SRC_RGA,
		AIO_AO_BURST_CLK_SRC_RGB,
		AIO_AO_BURST_CLK_SRC_EXT,
		AIO_AO_BURST_CLK_SRC_BCR,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_sscr_32, 7, 6);
	return(ret);
}



//*****************************************************************************
static int _ao_burst_enable(dev_data_t* dev, void* arg)
{
	static const s32	options[]	=
	{
		AIO_AO_BURST_ENABLE_NO,
		AIO_AO_BURST_ENABLE_YES,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bcr_32, 9, 9);
	return(ret);
}



//*****************************************************************************
static int _ao_clk_src(dev_data_t* dev, void* arg)
{
	static const s32	options[]	=
	{
		AIO_AO_CLK_SRC_RGA,
		AIO_AO_CLK_SRC_RGB,
		AIO_AO_CLK_SRC_EXT,
		AIO_AO_CLK_SRC_DISABLE,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_sscr_32, 5, 4);
	return(ret);
}



//*****************************************************************************
static int _ao_looping(dev_data_t* dev, void* arg)
{
	static const s32	options[]	=
	{
		AIO_AO_LOOPING_DISABLE,
		AIO_AO_LOOPING_ENABLE,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bcr_32, 10, 10);
	return(ret);
}



//*****************************************************************************
static int _ao_sync(dev_data_t* dev, void* arg)
{
	os_reg_mem_mx_u32(dev, dev->vaddr.gsc_bcr_32, D11, D11);
	return(0);
}



//*****************************************************************************
static int _ao_timing(dev_data_t* dev, void* arg)
{
	static const s32	options[]	=
	{
		AIO_AO_TIMING_SEQ,
		AIO_AO_TIMING_SIMUL,
		-1	// terminate list
	};

	int		ret	= 0;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bcr_32, 8, 8);
	return(ret);
}



//*****************************************************************************
static int _aux_read(dev_data_t* dev, void* arg)
{
	s32*	ptr	= (s32*) arg;
	int		ret;

	ptr[0]	= -1;
	ret		= gsc_s32_range_reg(dev, arg, 0, 1, dev->vaddr.gsc_diopr_32, 16, 16);
	return(ret);
}



//*****************************************************************************
static int _aux_write(dev_data_t* dev, void* arg)
{
	int	ret;

	ret	= gsc_s32_range_reg(dev, arg, 0, 1, dev->vaddr.gsc_diopr_32, 17, 17);
	return(ret);
}



//*****************************************************************************
static int _dio_dir_out(dev_data_t* dev, void* arg)
{
	int	ret;

	ret	= gsc_s32_range_reg(dev, arg, 0, 3, dev->vaddr.gsc_diopr_32, 19, 18);
	return(ret);
}



//*****************************************************************************
static int _dio_read(dev_data_t* dev, void* arg)
{
	s32*	ptr	= (s32*) arg;
	int		ret;

	ptr[0]	= -1;
	ret		= gsc_s32_range_reg(dev, arg, 0, 0xFFFF, dev->vaddr.gsc_diopr_32, 15, 0);
	return(ret);
}



//*****************************************************************************
static int _dio_write(dev_data_t* dev, void* arg)
{
	int	ret;

	ret	= gsc_s32_range_reg(dev, arg, 0, 0xFFFF, dev->vaddr.gsc_diopr_32, 15, 0);
	return(ret);
}



//*****************************************************************************
static int _irq0_sel(dev_data_t* dev, s32* arg)
{
	int	ret	= 0;
	u32	val;

	if (arg[0] == -1)
	{
		// Retrieve the current setting.
		val		= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_icr_32);
		arg[0]	= GSC_FIELD_DECODE(val, 2, 0);
	}
	else
	{
		// Validate the option value passed in.

		switch (arg[0])
		{
			default:

				ret	= -EINVAL;
				break;

			case AIO_IRQ0_IDLE_INIT:
			case AIO_IRQ0_AUTO_CAL_DONE:
			case AIO_IRQ0_AUX_IN_L2H:
			case AIO_IRQ0_AUX_IN_H2L:

				break;
		}

		if (ret == 0)
		{
			// Clear the status bit and apply the new setting.

			if (dev->cache.icr_irq_bug)
				val	= arg[0];
			else
				val	= arg[0] | 0x880;

			os_reg_mem_mx_u32(dev, dev->vaddr.gsc_icr_32, val, 0x88F);
		}
	}

	return(ret);
}



//*****************************************************************************
static int _irq1_sel(dev_data_t* dev, s32* arg)
{
	int	ret	= 0;
	u32	val;

	if (arg[0] == -1)
	{
		// Retrieve the current setting.
		val		= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_icr_32);
		arg[0]	= GSC_FIELD_DECODE(val, 6, 4);
	}
	else
	{
		// Validate the option value passed in.

		switch (arg[0])
		{
			default:

				ret	= -EINVAL;
				break;

			case AIO_IRQ1_IDLE:
			case AIO_IRQ1_AI_BUF_THR_L2H:
			case AIO_IRQ1_AI_BUF_THR_H2L:

				break;
		}

		if (ret == 0)
		{
			// Clear the status bit and apply the new setting.

			if (dev->cache.icr_irq_bug)
				val	= (arg[0] << 4);
			else
				val	= (arg[0] << 4) | 0x808;

			os_reg_mem_mx_u32(dev, dev->vaddr.gsc_icr_32, val, 0x8F8);
		}
	}

	return(ret);
}



//*****************************************************************************
static int _irq2_sel(dev_data_t* dev, s32* arg)
{
	int	ret	= 0;
	u32	val;

	if (arg[0] == -1)
	{
		// Retrieve the current setting.
		val		= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_icr_32);
		arg[0]	= GSC_FIELD_DECODE(val, 10, 8);
	}
	else
	{
		// Validate the option value passed in.

		switch (arg[0])
		{
			default:

				ret	= -EINVAL;
				break;

			case AIO_IRQ2_IDLE:
			case AIO_IRQ2_AO_BUF_THR_L2H:
			case AIO_IRQ2_AO_BUF_THR_H2L:
			case AIO_IRQ2_AO_BURST_DONE:

				break;
		}

		if (ret == 0)
		{
			// Clear the status bit and apply the new setting.

			if (dev->cache.icr_irq_bug)
				val	= (arg[0] << 8);
			else
				val	= (arg[0] << 8) | 0x088;

			os_reg_mem_mx_u32(dev, dev->vaddr.gsc_icr_32, val, 0xF88);
		}
	}

	return(ret);
}



//*****************************************************************************
static int _data_format(dev_data_t* dev, void* arg)
{
	static const s32	options[]	=
	{
		AIO_DATA_FORMAT_2S_COMP,
		AIO_DATA_FORMAT_OFF_BIN,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bcr_32, 6, 6);
	return(ret);
}



//*****************************************************************************
static int _ext_sync_src(dev_data_t* dev, void* arg)
{
	static const s32	options[]	=
	{
		AIO_AI_EXT_SYNC_SRC_AISC,
		AIO_AI_EXT_SYNC_SRC_AOS,
		AIO_AI_EXT_SYNC_SRC_EXT,
		AIO_AI_EXT_SYNC_SRC_DISABLE,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_sscr_32, 9, 8);
	return(ret);
}



//*****************************************************************************
static int _range(dev_data_t* dev, void* arg)
{
	static const s32	options[]	=
	{
		AIO_RANGE_2_5V,
		AIO_RANGE_5V,
		AIO_RANGE_10V,
		-1	// terminate list
	};

	u32		bcr;
	s32*	ptr	= (s32*) arg;
	int		ret	= 0;

	if (ptr[0] == -1)
	{
		bcr	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_bcr_32);

		switch (bcr & 0x30)
		{
			default:	ret	= -EINVAL;				break;
			case 0x00:	ptr[0]	= AIO_RANGE_2_5V;	break;
			case 0x10:	ptr[0]	= AIO_RANGE_5V;		break;
			case 0x20:
			case 0x30:	ptr[0]	= AIO_RANGE_10V;	break;
		}
	}
	else
	{
		ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bcr_32, 5, 4);
	}

	return(ret);
}



//*****************************************************************************
static int _rga_enable(dev_data_t* dev, void* arg)
{
	static const s32	options[]	=
	{
		AIO_GEN_ENABLE_NO,
		AIO_GEN_ENABLE_YES,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_rgar_32, 16, 16);
	return(ret);
}



//*****************************************************************************
static int _rga_nrate(dev_data_t* dev, void* arg)
{
	int	max;
	int	min;
	int	ret;

	_nrate_range(dev, &max, &min);
	ret	= gsc_s32_range_reg(dev, arg, min, max, dev->vaddr.gsc_rgar_32, 15, 0);
	return(ret);
}



//*****************************************************************************
static int _rgb_clk_src(dev_data_t* dev, void* arg)
{
	static const s32	options[]	=
	{
		AIO_RGB_CLK_SRC_MASTER,
		AIO_RGB_CLK_SRC_RGA,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_sscr_32, 10, 10);
	return(ret);
}



//*****************************************************************************
static int _rgb_enable(dev_data_t* dev, void* arg)
{
	static const s32	options[]	=
	{
		AIO_GEN_ENABLE_NO,
		AIO_GEN_ENABLE_YES,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_rgbr_32, 16, 16);
	return(ret);
}



//*****************************************************************************
static int _rgb_nrate(dev_data_t* dev, void* arg)
{
	int	max;
	int	min;
	int	ret;

	_nrate_range(dev, &max, &min);
	ret	= gsc_s32_range_reg(dev, arg, min, max, dev->vaddr.gsc_rgbr_32, 15, 0);
	return(ret);
}



//*****************************************************************************
static int _rx_io_mode(dev_data_t* dev, void* arg)
{
	static const s32	list[]	=
	{
		GSC_IO_MODE_PIO,
		GSC_IO_MODE_DMA,
		-1
	};

	int	ret;

	ret	= gsc_s32_list_var(arg, list, &dev->rx.io_mode);
	return(ret);
}



//*****************************************************************************
static int _rx_io_timeout(dev_data_t* dev, void* arg)
{
	int	ret;

	ret	= gsc_s32_range_var(
			arg,
			AIO_IO_TIMEOUT_MIN,
			AIO_IO_TIMEOUT_INFINITE,
			&dev->rx.timeout_s);
	return(ret);
}



//*****************************************************************************
static int _tx_io_mode(dev_data_t* dev, void* arg)
{
	static const s32	list[]	=
	{
		GSC_IO_MODE_PIO,
		GSC_IO_MODE_DMA,
		-1
	};

	int	ret;

	ret	= gsc_s32_list_var(arg, list, &dev->tx.io_mode);
	return(ret);
}



//*****************************************************************************
static int _tx_io_timeout(dev_data_t* dev, void* arg)
{
	int	ret;

	ret	= gsc_s32_range_var(
			arg,
			AIO_IO_TIMEOUT_MIN,
			AIO_IO_TIMEOUT_INFINITE,
			&dev->tx.timeout_s);
	return(ret);
}



//*****************************************************************************
static int _rx_io_abort(dev_data_t* dev, s32* arg)
{
	arg[0]	= gsc_read_abort_active_xfer(dev);
	return(0);
}



//*****************************************************************************
static int _tx_io_abort(dev_data_t* dev, s32* arg)
{
	arg[0]	= gsc_write_abort_active_xfer(dev);
	return(0);
}



// variables ******************************************************************

const gsc_ioctl_t	dev_ioctl_list[]	=
{
	{ AIO_IOCTL_REG_READ,			(void*) gsc_reg_read_ioctl		},
	{ AIO_IOCTL_REG_WRITE,			(void*) gsc_reg_write_ioctl		},
	{ AIO_IOCTL_REG_MOD,			(void*) gsc_reg_mod_ioctl		},
	{ AIO_IOCTL_QUERY,				(void*) _query					},
	{ AIO_IOCTL_INITIALIZE,			(void*) initialize_ioctl		},
	{ AIO_IOCTL_AUTO_CALIBRATE,		(void*) _auto_calibrate			},
	{ AIO_IOCTL_AUTO_CAL_STS,		(void*) _auto_cal_sts			},
	{ AIO_IOCTL_AI_BUF_CLEAR,		(void*) _ai_buf_clear			},
	{ AIO_IOCTL_AI_BUF_THR_LVL,		(void*) _ai_buf_thr_lvl			},
	{ AIO_IOCTL_AI_BUF_THR_STS,		(void*) _ai_buf_thr_sts			},
	{ AIO_IOCTL_AI_MODE,			(void*) _ai_mode				},
	{ AIO_IOCTL_AI_SCAN_1_CHAN,		(void*) _ai_scan_1_chan			},
	{ AIO_IOCTL_AI_SCAN_CLK_SRC,	(void*) _ai_scan_clk_src		},
	{ AIO_IOCTL_AI_SCAN_SIZE,		(void*) _ai_scan_size			},
	{ AIO_IOCTL_AI_SYNC,			(void*) _ai_sync				},
	{ AIO_IOCTL_AO_BUF_CLEAR,		(void*) _ao_buf_clear			},
	{ AIO_IOCTL_AO_BUF_THR_LVL,		(void*) _ao_buf_thr_lvl			},
	{ AIO_IOCTL_AO_BUF_THR_STS,		(void*) _ao_buf_thr_sts			},
	{ AIO_IOCTL_AO_BURST_CLK_SRC,	(void*) _ao_burst_clk_src		},
	{ AIO_IOCTL_AO_BURST_ENABLE,	(void*) _ao_burst_enable		},
	{ AIO_IOCTL_AO_CLK_SRC,			(void*) _ao_clk_src				},
	{ AIO_IOCTL_AO_LOOPING,			(void*) _ao_looping				},
	{ AIO_IOCTL_AO_SYNC,			(void*) _ao_sync				},
	{ AIO_IOCTL_AO_TIMING,			(void*) _ao_timing				},
	{ AIO_IOCTL_AUX_READ,			(void*) _aux_read				},
	{ AIO_IOCTL_AUX_WRITE,			(void*) _aux_write				},
	{ AIO_IOCTL_DIO_DIR_OUT,		(void*) _dio_dir_out			},
	{ AIO_IOCTL_DIO_READ,			(void*) _dio_read				},
	{ AIO_IOCTL_DIO_WRITE,			(void*) _dio_write				},
	{ AIO_IOCTL_IRQ0_SEL,			(void*) _irq0_sel				},
	{ AIO_IOCTL_IRQ1_SEL,			(void*) _irq1_sel				},
	{ AIO_IOCTL_IRQ2_SEL,			(void*) _irq2_sel				},
	{ AIO_IOCTL_DATA_FORMAT,		(void*) _data_format			},
	{ AIO_IOCTL_EXT_SYNC_SRC,		(void*) _ext_sync_src			},
	{ AIO_IOCTL_RANGE,				(void*) _range					},
	{ AIO_IOCTL_RGA_ENABLE,			(void*) _rga_enable				},
	{ AIO_IOCTL_RGA_NRATE,			(void*) _rga_nrate				},
	{ AIO_IOCTL_RGB_CLK_SRC,		(void*) _rgb_clk_src			},
	{ AIO_IOCTL_RGB_ENABLE,			(void*) _rgb_enable				},
	{ AIO_IOCTL_RGB_NRATE,			(void*) _rgb_nrate				},
	{ AIO_IOCTL_RX_IO_MODE,			(void*) _rx_io_mode				},
	{ AIO_IOCTL_RX_IO_TIMEOUT,		(void*) _rx_io_timeout			},
	{ AIO_IOCTL_TX_IO_MODE,			(void*) _tx_io_mode				},
	{ AIO_IOCTL_TX_IO_TIMEOUT,		(void*) _tx_io_timeout			},
	{ AIO_IOCTL_RX_IO_ABORT,		(void*) _rx_io_abort			},
	{ AIO_IOCTL_TX_IO_ABORT,		(void*) _tx_io_abort			},
	{ AIO_IOCTL_WAIT_EVENT,			(void*) gsc_wait_event_ioctl	},
	{ AIO_IOCTL_WAIT_CANCEL,		(void*) gsc_wait_cancel_ioctl	},
	{ AIO_IOCTL_WAIT_STATUS,		(void*) gsc_wait_status_ioctl	},
	{ -1, NULL }
};


