// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AIO/16AIO_Linux_4.x.x.x_DN/driver/irq.c $
// $Rev: 43208 $
// $Date: 2018-07-08 17:20:44 -0500 (Sun, 08 Jul 2018) $

// 16AIO: Device Driver: source file

#include "main.h"



//*****************************************************************************
void dev_irq_isr_local_handler(dev_data_t* dev)
{
	#define ICR_IRQ0_REQUEST	0x008
	#define ICR_IRQ1_REQUEST	0x080
	#define ICR_IRQ2_REQUEST	0x800

	u32	icr;
	u32	irq;

	icr	= os_reg_mem_rx_u32(NULL, dev->vaddr.gsc_icr_32);

	if (icr & ICR_IRQ0_REQUEST)
	{
		// Resume any blocked threads.
		irq	= GSC_FIELD_DECODE(icr, 2, 0);

		switch (irq)
		{
			default:

				gsc_wait_resume_irq_main(dev, GSC_WAIT_MAIN_SPURIOUS);
				break;

			case AIO_IRQ0_IDLE_INIT:

				gsc_wait_resume_irq_gsc(dev, AIO_WAIT_GSC_IDLE_INIT);
				break;

			case AIO_IRQ0_AUTO_CAL_DONE:

				gsc_wait_resume_irq_gsc(dev, AIO_WAIT_GSC_AUTO_CAL_DONE);
				break;

			case AIO_IRQ0_AUX_IN_L2H:

				gsc_wait_resume_irq_gsc(dev, AIO_WAIT_GSC_AUX_IN_L2H);
				break;

			case AIO_IRQ0_AUX_IN_H2L:

				gsc_wait_resume_irq_gsc(dev, AIO_WAIT_GSC_AUX_IN_H2L);
				break;
		}

		// Clear the interrupt.
		icr	= (icr & 0x777) | (dev->cache.icr_irq_bug ? 0 : 0x880);
		os_reg_mem_tx_u32(NULL, dev->vaddr.gsc_icr_32, icr);
	}
	else if (icr & ICR_IRQ1_REQUEST)
	{
		// Resume any blocked threads.
		irq	= GSC_FIELD_DECODE(icr, 6, 4);

		switch (irq)
		{
			default:

				gsc_wait_resume_irq_main(dev, GSC_WAIT_MAIN_SPURIOUS);
				break;

			case AIO_IRQ1_IDLE:

				dev->cache.icr_irq_bug	= 1;
				gsc_wait_resume_irq_main(dev, GSC_WAIT_MAIN_SPURIOUS);
				break;

			case AIO_IRQ1_AI_BUF_THR_L2H:

				gsc_wait_resume_irq_gsc(dev, AIO_WAIT_GSC_AI_BUF_THR_L2H);
				break;

			case AIO_IRQ1_AI_BUF_THR_H2L:

				gsc_wait_resume_irq_gsc(dev, AIO_WAIT_GSC_AI_BUF_THR_H2L);
				break;
		}

		// Clear the interrupt.
		icr	= (icr & 0x777) | (dev->cache.icr_irq_bug ? 0 : 0x808);
		os_reg_mem_tx_u32(NULL, dev->vaddr.gsc_icr_32, icr);
	}
	else if (icr & ICR_IRQ2_REQUEST)
	{
		// Resume any blocked threads.
		irq	= GSC_FIELD_DECODE(icr, 10, 8);

		switch (irq)
		{
			default:

				gsc_wait_resume_irq_main(dev, GSC_WAIT_MAIN_SPURIOUS);
				break;

			case AIO_IRQ2_IDLE:

				dev->cache.icr_irq_bug	= 1;
				gsc_wait_resume_irq_main(dev, GSC_WAIT_MAIN_SPURIOUS);
				break;

			case AIO_IRQ2_AO_BUF_THR_L2H:

				gsc_wait_resume_irq_gsc(dev, AIO_WAIT_GSC_AO_BUF_THR_L2H);
				break;

			case AIO_IRQ2_AO_BUF_THR_H2L:

				gsc_wait_resume_irq_gsc(dev, AIO_WAIT_GSC_AO_BUF_THR_H2L);
				break;

			case AIO_IRQ2_AO_BURST_DONE:

				gsc_wait_resume_irq_gsc(dev, AIO_WAIT_GSC_AO_BURST_DONE);
				break;
		}

		// Clear the interrupt.
		icr	= (icr & 0x777) | (dev->cache.icr_irq_bug ? 0 : 0x088);
		os_reg_mem_tx_u32(NULL, dev->vaddr.gsc_icr_32, icr);
	}
	else
	{
		// We don't know the source of the interrupt.
		gsc_wait_resume_irq_main(dev, GSC_WAIT_MAIN_SPURIOUS);
	}
}



//*****************************************************************************
int dev_irq_create(dev_data_t* dev)
{
	int	ret;

	os_reg_mem_tx_u32(dev, dev->vaddr.gsc_icr_32, 0);
	ret	= gsc_irq_create(dev);
	return(ret);
}



//*****************************************************************************
void dev_irq_destroy(dev_data_t* dev)
{
	if (dev->vaddr.gsc_icr_32)
		os_reg_mem_tx_u32(dev, dev->vaddr.gsc_icr_32, 0);

	gsc_irq_destroy(dev);
}


