// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AIO/16AIO_Linux_4.x.x.x_DN/driver/main.h $
// $Rev: 43208 $
// $Date: 2018-07-08 17:20:44 -0500 (Sun, 08 Jul 2018) $

// 16AIO: Device Driver: header file

#ifndef __MAIN_H__
#define __MAIN_H__

#include "16aio.h"

#define DEV_BAR_SHOW						0
#define DEV_METRICS_SHOW					0
#define DEV_PCI_ID_SHOW						0
#define DEV_SUPPORTS_IRQ
#define DEV_SUPPORTS_READ
#define DEV_SUPPORTS_WAIT
#define DEV_SUPPORTS_WRITE
#define GSC_ALT_DEV_GET(a)					(a)
#define GSC_ALT_STRUCT_GET(i,d)				(d)
#define GSC_ALT_STRUCT_T					dev_data_t
#define GSC_DEVS_PER_BOARD					1
#define	GSC_READ_PIO_PER_1MS				500				// metrics: ~2.0us per hw read on PCIe 1x
#define	GSC_WRITE_PIO_PER_1MS				7576			// metrics: ~132ns per hw write on PCIe 1x
typedef struct _dev_data_t					dev_data_t;

#include "gsc_main.h"



// #defines *******************************************************************

#define DEV_MODEL							"16AIO"	// Upper case form of the below.
#define DEV_NAME							"16aio"	// MUST AGREE WITH AIO_BASE_NAME

#define DEV_VERSION							"5.2"		// FOR DEVICE SPECIFIC CODE ONLY!
// 5.3	Changed "printk" calls to "printf" for cross platform compatibility.
// 5.2	Bug fix: was ignoring BAR setup return value.
//		Updated per changes to the OS specific PCI services.
//		Added metrics support.
//		Device open now succeeds even if initialization fails.
//		Removed redundant device I/O open code.
// 5.1	The BAR code has been updated to include the data under a single structure.
//		The register definitions have been updated.
//		I/O services now pass around an os_mem_t structure instead of a buffer pointer.
//		White space cleanup.
//		Changed the arbitrary wait event callback argument to a void* type.
//		Modified to allow multiple apps to simultaneously access a single device.
//		Correct access to BAR0 and BAR1 (is RO, was RW).
//		Added an infinite I/O timeout option.
//		Corrected the IOCTL code for the register write and mod services.
// 5.0	Updated to use the newer common driver sources.
//		Removed GNU notice from non-Linux specific files.
//		Removed Linux specific content from non-Linux specific source files.
//		White space cleanup.
//		Now using a spinlock rather than enabling and disabling interrupts.
//		Updated gsc_irq_open() and gsc_irq_close().
//		Updated gsc_dma_open() and gsc_dma_close().
//		Enhanced output reported when the DEV_PCI_ID_SHOW option is enabled.
//		Added an initialize post-delay.
// 4.6	Adjusted the minimum Nrate values to 2 to accommodate cascading.
// 4.5	Reduced #include list in driver interface header.
// 4.4	Updated for the 3.x kernel.
//		Corrected a system time tick count roll over bug in ioctl.c.
// 4.3	BAR0 and BAR2 are now the only BAR regions used.
//		Include all common source, though not all are used.
//		Implemented work around where write 1 to ICR status bits cause an IRQ.
// 4.2	Moved defines for D0-D31 to common code.
//		Eliminated global dev_check_id() routine.
// 4.1	Overhauled IOCTL based local interrupt implementation.
//		Updated per changes to the common source code.
//		Implemented IOCTL based I/O Abort support.
//		Implemented IOCTL based event/interrupt wait support.
//		Updated to use the common gsc_open() and gsc_close() services.
//		Implemented explicit local interrupt support.
//		Updated various Auto Cal related strings and macros for consistency.
//		Implemented support for the common PIO I/O routines.
//		Changed use of DEV_SUPPORTS_READ macro.
//		Changed use of DEV_SUPPORTS_WRITE macro.
// 4.0	Initial release of the overhauled driver.

// I/O services
#define DEV_PIO_READ_AVAILABLE				pio_read_available
#define DEV_PIO_READ_WORK					gsc_read_pio_work_32_bit
#define DEV_DMA_READ_AVAILABLE				dma_read_available
#define DEV_DMA_READ_WORK					dma_read_work
#define DEV_DMDMA_READ_AVAILABLE			dmdma_read_available
#define DEV_DMDMA_READ_WORK					dmdma_read_work

#define DEV_PIO_WRITE_AVAILABLE				pio_write_available
#define DEV_PIO_WRITE_WORK					gsc_write_pio_work_32_bit
#define DEV_DMA_WRITE_AVAILABLE				dma_write_available
#define DEV_DMA_WRITE_WORK					dma_write_work
#define DEV_DMDMA_WRITE_AVAILABLE			dmdma_write_available
#define DEV_DMDMA_WRITE_WORK				dmdma_write_work

#define GSC_READ_PIO_WORK_32_BIT
#define GSC_WRITE_PIO_WORK_32_BIT

// WAIT services
#define DEV_WAIT_GSC_ALL					AIO_WAIT_GSC_ALL
#define DEV_WAIT_ALT_ALL					AIO_WAIT_ALT_ALL



// data types *****************************************************************

struct _dev_io_t
{
	os_sem_t			sem;				// Only one Tx or Rx at a time.

	int					abort;
	int					bytes_per_sample;	// Sample size in bytes.
	gsc_dma_ch_t*		dma_channel;		// Use this channel for DMA.
	s32					io_mode;			// PIO, DMA, DMDMA
	u32					io_reg_offset;		// Offset of board's I/O FIFO.
	VADDR_T				io_reg_vaddr;		// Address of board's I/O FIFO.
	os_mem_t			mem;				// I/O buffer.
	int					non_blocking;		// Is this non-blocking I/O?
	s32					pio_threshold;		// Use PIO if samples <= this.
	s32					timeout_s;			// I/O timeout in seconds.
};

struct _dev_data_t
{
	os_pci_t			pci;			// The kernel PCI device descriptor.
	os_data_t			os;				// OS specific data.
	os_spinlock_t		spinlock;		// Control ISR access.
	os_sem_t			sem;			// Control thread access.
	gsc_dev_type_t		board_type;		// Corresponds to basic board type.
	const char*			model;			// Base model number in upper case.
	int					board_index;	// Index of the board being accessed.
	int					users;			// Number of currently active open() requests.
	int					share;			// Were we opened in shared mode?

	gsc_bar_t			bar;			// device register mappings
	gsc_dma_t			dma;			// For DMA based I/O.
	gsc_irq_t			irq;			// For interrut support.
	dev_io_t			rx;				// For read operations.
	dev_io_t			tx;				// For write operations.
	gsc_wait_node_t*	wait_list;

	struct
	{
		VADDR_T			plx_intcsr_32;	// Interrupt Control/Status Register
		VADDR_T			plx_dmaarb_32;	// DMA Arbitration Register
		VADDR_T			plx_dmathr_32;	// DMA Threshold Register

		VADDR_T			gsc_bcr_32;		// 0x00 Board Control Register
		VADDR_T			gsc_icr_32;		// 0x04 Interrupt Control Register
		VADDR_T			gsc_idbr_32;	// 0x08 Input Data Buffer Register
		VADDR_T			gsc_ibcr_32;	// 0x0C Input Buffer Control Register
		VADDR_T			gsc_rgar_32;	// 0x10 Rate Generator A Register
		VADDR_T			gsc_rgbr_32;	// 0x14 Rate Generator B Register
		VADDR_T			gsc_odbr_32;	// 0x18 Output Data Buffer Register
		VADDR_T			gsc_obcr_32;	// 0x1C Output Buffer Control Register
		VADDR_T			gsc_sscr_32;	// 0x20 Scan and Sync Control Register
		VADDR_T			gsc_diopr_32;	// 0x24 Digital I/O Port Register

	} vaddr;

	struct
	{
		int				auto_cal_ms;		// Maximum ms for auto-cal
		u32				fifo_size_rx;		// Size of Rx FIFO - not the fill level.
		u32				fifo_size_tx;		// Size of Tx FIFO - not the fill level.
		int				initialize_ms;		// Maximum ms for initialize

		s32				fgen_ai_max;		// Maximum Rate Generator Analog Input rate.
		s32				fgen_ai_min;		// Minimum Rate Generator Analog Input rate.
		s32				fgen_ao_max;		// Maximum Rate Generator Analog Output rate.
		s32				fgen_ao_min;		// Minimum Rate Generator Analog Output rate.
		s32				fref_default;		// Default reference clock rate.
		s32				fsamp_ai_max;		// Maximum Analog Input sample rate.
		s32				fsamp_ai_min;		// Minimum Analog Input sample rate.
		s32				fsamp_ao_max;		// Maximum Analog Output sample rate.
		s32				fsamp_ao_min;		// Minimum Analog Output sample rate.

		int				icr_irq_bug;		// Do writes of 1 to status bits caus an irq?

		int				nrate_ai_mask;		// Mask of valid Analog Input Nrate bits.
		int				nrate_ai_max;		// Minimum Analog Input Nrate value.
		int				nrate_ai_min;		// Maximum Analog Input Nrate value.
		int				nrate_ao_mask;		// Mask of valid Analog Input Nrate bits.
		int				nrate_ao_max;		// Minimum Analog Input Nrate value.
		int				nrate_ao_min;		// Maximum Analog Input Nrate value.

		int				res_bits;			// 12 or 16 bits of resolution

	} cache;
};



// prototypes *****************************************************************

void		dev_io_close(dev_data_t* dev);
int			dev_io_create(dev_data_t* dev);
void		dev_io_destroy(dev_data_t* dev);
int			dev_io_open(dev_data_t* dev);
int			dev_irq_create(dev_data_t* dev);
void		dev_irq_destroy(dev_data_t* dev);
long		dma_read_available(dev_data_t* dev, size_t samples);
long		dma_read_work(dev_data_t* dev, const os_mem_t* mem, size_t count, os_time_tick_t st_end);
long		dma_write_available(dev_data_t* dev, size_t samples);
long		dma_write_work(dev_data_t* dev, const os_mem_t* mem, size_t count, os_time_tick_t st_end);
long		dmdma_read_available(dev_data_t* dev, size_t samples);
long		dmdma_read_work(dev_data_t* dev, const os_mem_t* mem, size_t count, os_time_tick_t st_end);
long		dmdma_write_available(dev_data_t* dev, size_t samples);
long		dmdma_write_work(dev_data_t* dev, const os_mem_t* mem, size_t count, os_time_tick_t st_end);

int			initialize_ioctl(dev_data_t* dev, void* arg);

long		pio_read_available(dev_data_t* dev, size_t samples);
long		pio_write_available(dev_data_t* dev, size_t samples);



#endif
