// $URL: http://subversion:8080/svn/gsc/trunk/drivers/gsc_common/driver/intime/os_main.c $
// $Rev: 43205 $
// $Date: 2018-07-08 17:01:59 -0500 (Sun, 08 Jul 2018) $

// INtime: Device Driver: source file

#include "main.h"



// variables ******************************************************************

static	int	_exit	= 0;
static	int	_help	= 0;



//*****************************************************************************
static void _parse_args(int argc, char** argv)
{
	char	c;
	int		i;
	int		j;
	long	k;

	for (i = 1; i < argc; i++)
	{
		if (strcmp(argv[i], "-h") == 0)
		{
			_help	= 1;
			continue;
		}

		if (strcmp(argv[i], "-v") == 0)
		{
			gsc_global.verbose	= 1;
			continue;
		}

		if (strcmp(argv[i], "-x") == 0)
		{
			_exit	= 1;
			continue;
		}

		if ((argv[i][0] == '-') &&
			(argv[i][1] == 'p') &&
			(argv[i][2]))
		{
			j	= sscanf(&argv[i][2], "%ld%c", &k, &c);

			if ((j == 1) && (k >= 131) && (k <= 250))
			{
				printf("%s: Adjusting thread priorities to: %d\n", DEV_NAME, (int) k);
				gsc_global.priority	= (int) k;
				continue;
			}
		}

		_help	= 1;
		printf("ERROR: invalid argument: %s\n", argv[i]);
		break;
	}
}



//*****************************************************************************
static void _usage(void)
{
	printf("USAGE: 16aio <-h> <-p#> <-v> <-x>\n");
	printf("   -h    Show this usage information.\n");
	printf("   -p#   Set thread priorities (131-250, default is 131).\n");
	printf("   -v    Generate verbose output. Useful mostly for debugging.\n");
	printf("   -x    Tell the driver to exit.\n");
}


//*****************************************************************************
void main(int argc, char* argv[])
{
	int	ret;

#ifdef __OS_VALIDATE__

		os_validate();				// Validate data types and macros, then exit.

#endif

	gsc_global.priority	= 131;

	_parse_args(argc, argv);

	if (_help)
	{
		_usage();
	}
	else if (_exit)
	{
		gsc_global.verbose	= 1;
		printf("Initiating driver exit ... ");
		ret	= os_exit_initiate("\n");

		if (ret == 0)
			printf("Done\n");
	}
	else
	{
		printf("%s: driver loading: version %s\n", DEV_NAME, GSC_DRIVER_VERSION);
		ret	= os_mbox_driver_exists();

		if (ret == 0)
		{
			ret	= os_init_driver();

			if (ret)
				db_printf("%s: Failed driver initialization.\n", DEV_NAME);
			else		
				os_mbox_driver_start();

			os_exit_driver();
			os_exit_initiate("");
		}
	}
}


