// $URL: http://subversion:8080/svn/gsc/trunk/drivers/gsc_common/driver/intime/os_validate.c $
// $Rev: 43205 $
// $Date: 2018-07-08 17:01:59 -0500 (Sun, 08 Jul 2018) $

// INtime: Device Driver: source file

#include "main.h"

#ifdef __OS_VALIDATE__



//*****************************************************************************
static void _msg_stat_entry(const char* field, int f_len, const char* macro, int actual, int correct)
{
	if (field == NULL)
	{
		printf("\n");
		printf("Data Item        Len  Exp  Macro                     Act  Status\n");
		printf("===============  ===  ===  ========================  ===  ======\n");
	}
	else
	{
		printf("%-15s", field);

		if (f_len >= 0)
			printf("  %3d", f_len);
		else
			printf("     ");

		if (correct >= 0)
			printf("  %3d", correct);
		else
			printf("     ");

		printf("  %-24s", macro);

		if (actual >= 0)
			printf("  %3d", actual);
		else
			printf("     ");

		if ((correct >= 0) &&  (actual >= 0))
			printf("  %s\n", (correct == actual) ? "PASS" : "FAIL <---");
		else
			printf("\n");
	}
}



//*****************************************************************************
static void _msg_stats(void)
{
	os_msg_t	msg;
	long		msg_all			= sizeof(msg);
	long		msg_common		= (long) &msg.u - (long) &msg;
	long		msg_u			= sizeof(msg.u);
	long		msg_open		= sizeof(msg.u.open);
	long		msg_close		= sizeof(msg.u.close);
	long		msg_ioctl_sm	= sizeof(msg.u.ioctl_sm);
	long		msg_ioctl_sm_bf	= sizeof(msg.u.ioctl_sm.buf);
	long		msg_ioctl_lg	= sizeof(msg.u.ioctl_lg);
	long		msg_read		= sizeof(msg.u.read);
	long		msg_write		= sizeof(msg.u.write);
	long		msg_exit		= sizeof(msg.u.exit);

	_msg_stat_entry(NULL, 0, NULL, 0, 0);

	_msg_stat_entry("msg",				msg_all,		"",							-1,							-1);
	_msg_stat_entry("msg.u",			msg_u,			"",							-1,							-1);
	_msg_stat_entry("msg-common",		msg_common,		"OS_MSG_SIZE_COMMON",		OS_MSG_SIZE_COMMON,			msg_common);
	_msg_stat_entry("msg.u.open",		msg_open,		"OS_MSG_SIZE_OPEN",			OS_MSG_SIZE_OPEN,			msg_common + msg_open);
	_msg_stat_entry("msg.u.close",		msg_close,		"OS_MSG_SIZE_CLOSE",		OS_MSG_SIZE_CLOSE,			msg_common + msg_close);
	_msg_stat_entry("msg.u.exit",		msg_exit,		"OS_MSG_SIZE_EXIT",			OS_MSG_SIZE_EXIT,			msg_common + msg_exit);
	_msg_stat_entry("msg.u.read",		msg_read,		"OS_MSG_SIZE_READ",			OS_MSG_SIZE_READ,			msg_common + msg_read);
	_msg_stat_entry("msg.u.write",		msg_write,		"OS_MSG_SIZE_WRITE",		OS_MSG_SIZE_WRITE,			msg_common + msg_write);
	_msg_stat_entry("msg.u.ioctl_lg",	msg_ioctl_lg,	"OS_MSG_SIZE_IOCTL_LG",		OS_MSG_SIZE_IOCTL_LG,		msg_common + msg_ioctl_lg);
	_msg_stat_entry("msg.u.ioctl_sm",	msg_ioctl_sm,	"OS_MSG_SIZE_IOCTL_SM_MAX",	OS_MSG_SIZE_IOCTL_SM_MAX,	msg_common + msg_ioctl_sm);
	_msg_stat_entry("",					-1,				"OS_MSG_SIZE_IOCTL_SM_MIN",	OS_MSG_SIZE_IOCTL_SM_MIN,	msg_common + msg_ioctl_sm - msg_ioctl_sm_bf);
}



//*****************************************************************************
static void _data_type_entry(const char* type, size_t size)
{
	if (type == NULL)
	{
		printf("\n");
		printf("Data Type  Len\n");
		printf("=========  ===\n");
	}
	else
	{
		printf("%-9s", type);

		if (size >= 0)
			printf("  %3d", size);
		else
			printf("     ");

		printf("\n");
	}
}



//*****************************************************************************
static void _data_type_sizes(void)
{
	_data_type_entry(NULL,		0);
	_data_type_entry("char",	sizeof(char));
	_data_type_entry("short",	sizeof(short));
	_data_type_entry("int",		sizeof(int));
	_data_type_entry("long",	sizeof(long));
	_data_type_entry("BYTE",	sizeof(BYTE));
	_data_type_entry("WORD",	sizeof(WORD));
	_data_type_entry("DWORD",	sizeof(DWORD));
	_data_type_entry("QWORD",	sizeof(QWORD));
	_data_type_entry("void*",	sizeof(void*));
}



//*****************************************************************************
void os_validate(void)
{
	_msg_stats();

	_data_type_sizes();

	exit(0);
}


#endif
