// $URL:$
// $Rev:$
// $Date:$

// 16AIO: Sample Application: source file

#include "main.h"



// variables ******************************************************************

static	u32	_buf[0x7FFF];



//*****************************************************************************
static int _compute_channels(int fd, s32 scan_size, int* chans)
{
	int	errs	= 0;

	gsc_label("Channels");

	switch (scan_size)
	{
		default:						errs++;
		case AIO_AI_SCAN_SIZE_SINGLE:	chans[0]	= 1;	break;
		case AIO_AI_SCAN_SIZE_0_1:		chans[0]	= 2;	break;
		case AIO_AI_SCAN_SIZE_0_3:		chans[0]	= 4;	break;
		case AIO_AI_SCAN_SIZE_0_7:		chans[0]	= 8;	break;
		case AIO_AI_SCAN_SIZE_0_15:		chans[0]	= 16;	break;
		case AIO_AI_SCAN_SIZE_0_31:		chans[0]	= 32;	break;
	}

	if (errs)
	{
		printf("FAIL <---  (defaulting to one channel)\n");
	}
	else
	{
		printf("PASS  (%ld channel%s)\n", (long) chans[0], (chans[0] == 1) ? "" :"s");
	}

	return(errs);
}



//*****************************************************************************
static int _config(int fd, s32 fsamp, s32 scan_size, s32 threshold, s32 io_mode)
{
	int		errs;
	s32		chans;
	s32		nrate;
	double	rate;

	gsc_label("Setup");
	printf("\n");
	gsc_label_level_inc();

	gsc_label("Configuration");
	errs	= aio_config_ai(fd, -1, 0, fsamp);
	printf("%s\n", errs ? "FAIL <---" : "PASS");

	errs	+= aio_rx_io_mode			(fd, -1, 1, io_mode, NULL);
	errs	+= aio_ai_mode				(fd, -1, 1, AIO_AI_MODE_SINGLE, NULL);
	errs	+= aio_ai_scan_size			(fd, -1, 1, scan_size, NULL);
	errs	+= aio_ai_buf_thr_lvl		(fd, -1, 1, threshold, NULL);
	errs	+= aio_fsamp_ai_compute		(fd, -1, 1, fsamp, &nrate, &rate);
	errs	+= _compute_channels		(fd, scan_size, &chans);

	gsc_label("Overall Sample Rate");
	gsc_label_float_comma(rate * chans, -1, 3);
	printf(" S/S\n");

	gsc_label("Compute Wait Rate");
	gsc_label_float_comma(rate * chans / threshold, -1, 3);
	printf(" W/S\n");

	gsc_label_level_dec();
	return(errs);
}



//*****************************************************************************
static void _show_rate(const char* title, const char* abrev, long long us, long count)
{
	long long	ns;

	if (count)
		ns	= us * 1000 / count;
	else
		ns	= 1;

	if (ns == 0)
		ns	= 1;

	gsc_label(title);

	if (ns < 1000000)
	{
		gsc_label_long_comma(ns / 1000);
		printf(".%03ld us/%s (ave)\n", (long) (ns % 1000), abrev);
	}
	else if (ns < 1000000000)
	{
		gsc_label_long_comma(ns / 1000000);
		printf(".%03ld ms/%s (ave)\n", (long) ((ns / 1000) % 1000), abrev);
	}
	else
	{
		gsc_label_long_comma(ns / 1000000000);
		printf(".%06ld S/%s (ave)\n", (long) ((ns / 1000) % 1000000), abrev);
	}

}



//*****************************************************************************
static int _reset_threshold_status(int fd, int threshold, int reset_method, u32* dst, long long* reset_us)
{
	s32				bytes;
	int				errs	= 0;
	int				i;
	gsc_reg_t		idbr;
	struct timeval	reset_begin;
	struct timeval	reset_end;
	int				ret;

	idbr.reg	= AIO_GSC_IDBR;
	gettimeofday(&reset_begin, NULL);

	if (reset_method == RESET_METHOD_CLEAR)
	{
		// It takes a while to clear the buffer.
		ret		= aio_ioctl(fd, AIO_IOCTL_AI_BUF_CLEAR, NULL);
		errs	+= ret ? 1 : 0;
	}
	else if (reset_method == RESET_METHOD_REG_READS)
	{
		for (i = 0; i < threshold; i++)
		{
			ret		= aio_ioctl(fd, AIO_IOCTL_REG_READ, &idbr);
			errs	+= ret ? 1 : 0;
			dst[i]	= idbr.value;
		}
	}
	else if (reset_method == RESET_METHOD_API_READS)
	{
		bytes	= threshold * 4;
		ret		= aio_read(fd, dst, bytes);
		errs	+= (ret != bytes) ? 1 : 0;
	}
	else // if (reset_method == RESET_METHOD_IGNORE)
	{
	}

	gettimeofday(&reset_end, NULL);
	reset_us[0]	+= (long long) reset_end.tv_sec * 1000000 + reset_end.tv_usec;
	reset_us[0]	-= (long long) reset_begin.tv_sec * 1000000 + reset_begin.tv_usec;

	return(errs);
}



//*****************************************************************************
static int _test_method_poll(int fd, int threshold, int period, int reset_method)
{
	struct timeval	begin;
	long			detections;
	struct timeval	end;
	int				errs		= 0;
	gsc_reg_t		ibcr;
	u32				ibcr_last	= 0;
	int				missed		= 0;
	struct timeval	now;
	long			polls;
	long long		reset_us	= 0;
	int				ret;
	long long		us;

	if (reset_method == RESET_METHOD_CLEAR)
		gsc_label("POLL/Clear");
	else if (reset_method == RESET_METHOD_REG_READS)
		gsc_label("POLL/Reg Reads");
	else if (reset_method == RESET_METHOD_API_READS)
		gsc_label("POLL/API aio_read()");
	else // if (reset_method == RESET_METHOD_IGNORE)
		gsc_label("POLL/Ignore");

	fflush(stdout);
	memset(&ibcr, 0, sizeof(ibcr));
	ibcr.reg	= AIO_GSC_IBCR;

	errs	+= aio_ai_buf_clear(fd, -1, 0);
	gettimeofday(&begin, NULL);			// Time is now critical. Make direct calls.
	end.tv_sec	= begin.tv_sec + period;
	end.tv_usec	= begin.tv_usec;
	detections	= 0;
	ibcr_last	= 0;
	polls		= 0;

	for (;;)
	{
		polls++;
		ret		= aio_ioctl(fd, AIO_IOCTL_REG_READ, &ibcr);
		errs	+= ret ? 1 : 0;

		if (ibcr.value & 0x10000)
		{
			if (ibcr_last & 0x10000)
				missed++;

			detections++;
			errs	+= _reset_threshold_status(fd, threshold, reset_method, _buf, &reset_us);
		}

		ibcr_last	= ibcr.value;
		gettimeofday(&now, NULL);		// Time is no longer critical.

		if ((now.tv_sec > end.tv_sec) ||
			((now.tv_sec == end.tv_sec) && (now.tv_usec >= end.tv_usec)))
		{
			break;
		}
	}

	printf("Done\n");

	us	= ((long long) now.tv_sec * 1000000 + now.tv_usec)
		- ((long long) begin.tv_sec * 1000000 + begin.tv_usec);

	gsc_label("Period");
	gsc_label_long_comma(us / 1000000);
	printf(".%06ld Seconds\n", (long) (us % 1000000));

	gsc_label("POLLs");
	gsc_label_long_comma(polls);
	printf("\n");

	_show_rate("POLL Interval", "P", us, polls);

	gsc_label("Detections");
	gsc_label_long_comma(detections);
	printf("\n");

	_show_rate("Detection Interval", "D", us, detections);
	_show_rate("Reset Time", "R", reset_us, detections);

	gsc_label("Missed");
	gsc_label_long_comma(missed);
	printf("\n");

	gsc_label("Errors");
	gsc_label_long_comma(errs);
	printf("\n");

	gsc_label_level_dec();
	return(errs);
}



//*****************************************************************************
static int _test_method_wait(int fd, int threshold, int period, int reset_method)
{
	struct timeval	begin;
	long			detected;
	struct timeval	end;
	int				errs		= 0;
	s32				irq;
	struct timeval	now;
	long long		reset_us	= 0;
	int				ret;
	long			timeouts;
	long long		us;
	gsc_wait_t		wait;
	long			waits;


	if (reset_method == RESET_METHOD_CLEAR)
		gsc_label("WAIT/Clear");
	else if (reset_method == RESET_METHOD_REG_READS)
		gsc_label("WAIT/Reg Reads");
	else if (reset_method == RESET_METHOD_API_READS)
		gsc_label("WAIT/API aio_read()");
	else // if (reset_method == RESET_METHOD_IGNORE)
		gsc_label("WAIT/Ignore");

	fflush(stdout);

	aio_ai_buf_clear(fd, -1, 0);
	gettimeofday(&begin, NULL);			// Time is now critical. Make direct calls.

	irq		= AIO_IRQ1_AI_BUF_THR_L2H;
	ret		= aio_ioctl(fd, AIO_IOCTL_IRQ1_SEL, &irq);
	errs	+= ret ? 1 : 0;

	end.tv_sec	= begin.tv_sec + period;
	end.tv_usec	= begin.tv_usec;
	detected	= 0;
	timeouts	= 0;
	waits		= 0;

	for (;;)
	{
		waits++;
		memset(&wait, 0, sizeof(wait));
		wait.gsc		= AIO_WAIT_GSC_AI_BUF_THR_L2H;
		wait.timeout_ms	= 1000;
		ret				= aio_ioctl(fd, AIO_IOCTL_WAIT_EVENT, &wait);
		errs			+= ret ? 1 : 0;

		if (wait.flags & GSC_WAIT_FLAG_DONE)
			detected++;

		if (wait.flags & GSC_WAIT_FLAG_TIMEOUT)
			timeouts++;

		_reset_threshold_status(fd, threshold, reset_method, _buf, &reset_us);

		gettimeofday(&now, NULL);		// Time is no longer critical.

		if ((now.tv_sec > end.tv_sec) ||
			((now.tv_sec == end.tv_sec) && (now.tv_usec >= end.tv_usec)))
		{
			break;
		}
	}

	printf("Done\n");

	us	= ((long long) now.tv_sec * 1000000 + now.tv_usec)
		- ((long long) begin.tv_sec * 1000000 + begin.tv_usec);

	gsc_label("Period");
	gsc_label_long_comma(us / 1000000);
	printf(".%06ld Seconds\n", (long) (us % 1000000));

	gsc_label("WAITs");
	gsc_label_long_comma(waits);
	printf("\n");

	_show_rate("WAIT Interval", "P", us, waits);

	gsc_label("Detections");
	gsc_label_long_comma(detected);
	printf("\n");

	_show_rate("Detection Interval", "D", us, detected);
	_show_rate("Reset Time", "R", reset_us, waits);

	gsc_label("Timeouts");
	gsc_label_long_comma(timeouts);
	printf("\n");

	gsc_label("Errors");
	gsc_label_long_comma(errs);
	printf("\n");

	gsc_label_level_dec();
	return(errs);
}



//*****************************************************************************
static int _process(int fd, int errs, int threshold, int period, int monitor_method, int reset_method)
{
	gsc_label("Processing");

	if (errs)
	{
		printf("ABORTED DUE TO ERRORS\n");
		errs	= 0;
	}
	else
	{
		printf("\n");
		gsc_label_level_inc();

		gsc_label("Work Period");
		gsc_label_long_comma(period);
		printf(" Seconds\n");

		if (monitor_method == MONITOR_METHOD_POLE)
		{
			errs	= _test_method_poll(fd, threshold, period, reset_method);
		}
		else
		{
			errs	= _test_method_wait(fd, threshold, period, reset_method);
		}

		gsc_label_level_dec();
	}

	return(errs);
}



//*****************************************************************************
int ai_threshold(int fd, s32 fsamp, s32 scan_size, s32 threshold, int period, int monitor_method, int reset_method, s32 io_mode)
{
	int	errs	= 0;

	errs	+= _config(fd, fsamp, scan_size, threshold, io_mode);
	errs	+= _process(fd, errs, threshold, period, monitor_method, reset_method);

	return(errs);
}


