// $URL:$
// $Rev:$
// $Date:$

// 16AIO: Sample Application: source file

#include "main.h"



// #defines *******************************************************************

#define	FSAMP				10000
#define	PERIOD				10
#define	THRESHOLD			160



// variables ******************************************************************

static	int		_continuous		= 0;
static	int		_def_index		= 0;
static	s32		_fsamp			= FSAMP;
static	int		_ignore_errors	= 0;
static	s32		_io_mode		= GSC_IO_MODE_PIO;
static	long	_minute_limit	= 0;
static	int		_monitor_method	= MONITOR_METHOD_POLE;
static	int		_period			= PERIOD;
static	int		_reset_method	= RESET_METHOD_CLEAR;
static	s32		_scan_size		= AIO_AI_SCAN_SIZE_0_15;
static	int		_test_limit		= 0;
static	s32		_threshold		= THRESHOLD;



//*****************************************************************************
static int _parse_args(int argc, char** argv)
{
	char	buf[64];
	char	c;
	int		errs	= 0;
	int		i;
	int		j;
	long	k;

	printf("aithwaitrate - Analog Input WAIT/Rate Sample\n");
	printf("USAGE: aithwaitrate <-#> <-c> <-C> <-dma> <-m#> <-mp> <-mw> <-n#> <-p#> <-pio> <-ra> <-rc> <-rc> <-rr> <-sr#> <-th#> <index>\n");
	printf("  -#      Enter 1, 2, 4, 8, 16 or 32 for number channels to use.\n");
	printf("  -dma    Use Block bode mMA with the -ra option.\n");
	printf("  -c      Continue testing until an error occurs.\n");
	printf("  -C      Continue testing even if errors occur.\n");
	printf("  -m#     Run for at most # minutes (a decimal number).\n");
	printf("  -mp     Monitor the Thresold Status flag by POLLing (default, don't use WAIT).\n");
	printf("  -mw     Monitor the Threshold Status flag via the WAIT service (don't POLL)\n");
	printf("  -n#     Repeat test at most # times (a decimal number).\n");
	printf("  -p#     Work for a period of this number of seconds (default is %d).\n", PERIOD);
	printf("  -pio    Use PIO with the -ra option (default).\n");
	printf("  -ra     Reset the Threshold Status by API aio_read() requests.\n");
	printf("  -rc     Reset the Threshold Status flag by clearing the buffer (default).\n");
	printf("  -ri     Ignore the condition and do nothing.\n");
	printf("  -rr     Reset the Threshold Status flag by -th# number of register reads.\n");
	printf("  -sr#    Configure the board for this Sample Rate (default is %d).\n", FSAMP);
	printf("  -th#    Set the Input Buffer Threshold to this value (default is %d).\n", THRESHOLD);
	printf("  index   The zero based index of the device to access.\n");

	gsc_label("Arguments");
	printf("(%d argument%s)\n", argc - 1, ((argc - 1) == 1) ? "s" : "");
	gsc_label_level_inc();

	for (i = 0; i < argc; i++)
	{
		sprintf(buf, "Argument %d", i);
		gsc_label(buf);
		printf("%s\n", argv[i]);
	}

	gsc_label_level_dec();

	for (i = 1; i < argc; i++)
	{
		if (strcmp(argv[i], "-1") == 0)
		{
			_scan_size	= AIO_AI_SCAN_SIZE_SINGLE;
			continue;
		}

		if (strcmp(argv[i], "-2") == 0)
		{
			_scan_size	= AIO_AI_SCAN_SIZE_0_1;
			continue;
		}

		if (strcmp(argv[i], "-4") == 0)
		{
			_scan_size	= AIO_AI_SCAN_SIZE_0_3;
			continue;
		}

		if (strcmp(argv[i], "-8") == 0)
		{
			_scan_size	= AIO_AI_SCAN_SIZE_0_7;
			continue;
		}

		if (strcmp(argv[i], "-16") == 0)
		{
			_scan_size	= AIO_AI_SCAN_SIZE_0_15;
			continue;
		}

		if (strcmp(argv[i], "-32") == 0)
		{
			_scan_size	= AIO_AI_SCAN_SIZE_0_31;
			continue;
		}

		if (strcmp(argv[i], "-c") == 0)
		{
			_continuous		= 1;
			_ignore_errors	= 0;
			continue;
		}

		if (strcmp(argv[i], "-C") == 0)
		{
			_continuous		= 1;
			_ignore_errors	= 1;
			continue;
		}

		if (strcmp(argv[i], "-dma") == 0)
		{
			_io_mode	= GSC_IO_MODE_DMA;
			continue;
		}

		if ((argv[i][0] == '-') &&
			(argv[i][1] == 'm') &&
			(argv[i][2]))
		{
			j	= sscanf(&argv[i][2], "%ld%c", &k, &c);

			if ((j == 1) && (k > 0))
			{
				_minute_limit	= k;
				continue;
			}
		}

		if (strcmp(argv[i], "-mp") == 0)
		{
			_monitor_method	= MONITOR_METHOD_POLE;
			continue;
		}

		if (strcmp(argv[i], "-mw") == 0)
		{
			_monitor_method	= MONITOR_METHOD_WAIT;
			continue;
		}

		if ((argv[i][0] == '-') &&
			(argv[i][1] == 'n') &&
			(argv[i][2]))
		{
			j	= sscanf(&argv[i][2], "%ld%c", &k, &c);

			if ((j == 1) && (k > 0))
			{
				_test_limit	= (int) k;
				continue;
			}
		}

		if ((argv[i][0] == '-') &&
			(argv[i][1] == 'p') &&
			(argv[i][2]))
		{
			j	= sscanf(&argv[i][2], "%ld%c", &k, &c);

			if ((j == 1) && (k > 0) && (k <= 3600))
			{
				_period	= k;
				continue;
			}
		}

		if (strcmp(argv[i], "-pio") == 0)
		{
			_io_mode	= GSC_IO_MODE_PIO;
			continue;
		}

		if (strcmp(argv[i], "-ra") == 0)
		{
			_reset_method	= RESET_METHOD_API_READS;
			continue;
		}

		if (strcmp(argv[i], "-rc") == 0)
		{
			_reset_method	= RESET_METHOD_CLEAR;
			continue;
		}

		if (strcmp(argv[i], "-ri") == 0)
		{
			_reset_method	= RESET_METHOD_IGNORE;
			continue;
		}

		if (strcmp(argv[i], "-rr") == 0)
		{
			_reset_method	= RESET_METHOD_REG_READS;
			continue;
		}

		if ((argv[i][0] == '-') &&
			(argv[i][1] == 's') &&
			(argv[i][2] == 'r') &&
			(argv[i][3]))
		{
			j	= sscanf(&argv[i][3], "%ld%c", &k, &c);

			if ((j == 1) && (k > 0))
			{
				_fsamp	= k;
				continue;
			}
		}

		if ((argv[i][0] == '-') &&
			(argv[i][1] == 't') &&
			(argv[i][2] == 'h') &&
			(argv[i][3]))
		{
			j	= sscanf(&argv[i][3], "%ld%c", &k, &c);

			if ((j == 1) && (k > 0))
			{
				_threshold	= k;
				continue;
			}
		}

		j	= sscanf(argv[i], "%ld%c", &k, &c);

		if ((j == 1) && (k >= 0))
		{
			_def_index	= (int) k;
			continue;
		}

		errs	= 1;
		printf("ERROR: invalid argument: #%d = '%s'\n", i, argv[i]);
		break;
	}

	return(errs);
}



//*****************************************************************************
static int _perform_tests(int fd)
{
	int	errs	= 0;

	errs	+= ai_threshold(fd, _fsamp, _scan_size, _threshold, _period, _monitor_method, _reset_method, _io_mode);

	return(errs);
}



//*****************************************************************************
static void _show_access_index(int index)
{
	gsc_label("Accessing Device");
	printf("%d\n", index);
}



//*****************************************************************************
static void _show_time_stamp(void)
{
	const char*	psz;
	struct tm*	stm;
	time_t		tt;

	time(&tt);
	stm	= localtime(&tt);
	psz	= (char*) asctime(stm);
	gsc_label("Time Stamp");
	printf("%s", psz);
}



/******************************************************************************
*
*	Function:	main
*
*	Purpose:
*
*		Control the overall flow of the application.
*
*	Arguments:
*
*		argc			The number of command line arguments.
*
*		argv			The list of command line arguments.
*
*	Returned:
*
*		EXIT_SUCCESS	No errors were encounterred.
*		EXIT_FAILURE	One or more problems were encounterred.
*
******************************************************************************/

int main(int argc, char** argv)
{
	int		errs;
	time_t	exec		= time(NULL);
	long	failures	= 0;
	int		fd			= 0;
	long	hours;
	long	mins;
	time_t	now;
	long	passes		= 0;
	int		qty			= 0;
	int		ret;
	long	secs;
	time_t	t_limit;
	time_t	test;
	long	tests		= 0;

	for (;;)
	{
		gsc_label_init(22);
		test	= time(NULL);
		errs	= _parse_args(argc, argv);

		if (errs)
			break;

		// Introduction

		t_limit	= exec + (_minute_limit * 60);
		os_id_host();
		_show_time_stamp();
		_show_access_index(_def_index);
		errs	+= aio_init_util(1);
		errs	+= os_id_driver(aio_open, aio_read, aio_close);
		errs	+= aio_count_boards(1, &qty);

		if ((errs) || (qty <= 0))
			break;

		// Open access to device

		errs	= aio_open_util(_def_index, 0, -1, 1, &fd);

		if (errs == 0)
		{
			// Device identification

			errs	= aio_id_device(fd, -1, 1);

			// Perform testing

			errs	+= _perform_tests(fd);

			// Close device access

			errs	+= aio_close_util(fd, -1, 1);
		}

		// End of processing.

		now	= time(NULL);
		tests++;

		if (errs)
		{
			failures++;
			printf(	"\nRESULTS: FAIL <---  (%d error%s)",
					errs,
					(errs == 1) ? "" : "s");
		}
		else
		{
			passes++;
			printf("\nRESULTS: PASS");
		}

		secs	= (long) now - test;
		hours	= secs / 3600;
		secs	= secs % 3600;
		mins	= secs / 60;
		secs	= secs % 60;
		printf(" (duration %ld:%ld:%02ld)\n", hours, mins, secs);

		secs	= now - exec;
		hours	= secs / 3600;
		secs	= secs % 3600;
		mins	= secs / 60;
		secs	= secs % 60;
		printf(	"SUMMARY: tests %ld, pass %ld, fail %ld"
				" (duration %ld:%ld:%02ld)\n\n",
				tests,
				passes,
				failures,
				hours,
				mins,
				secs);

		if ((_test_limit > 0) && (_test_limit <= tests))
			break;

		if (_continuous == 0)
			break;

		if ((_ignore_errors == 0) && (errs))
			break;

		if ((_minute_limit) && (now >= t_limit))
			break;
	}

	ret	= ((qty <= 0) || (errs) || (failures)) ? EXIT_FAILURE : EXIT_SUCCESS;
	return(ret);
}


