// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AIO/16AIO_Linux_4.x.x.x_DN/samples/aout/aio.c $
// $Rev: 43220 $
// $Date: 2018-07-08 17:47:32 -0500 (Sun, 08 Jul 2018) $

// 16AIO: Sample Application: source file

#include "main.h"



// variables ******************************************************************

static	u32		_data[128 * 1024];
static	long	_loops;	// Number of times to send a block of data.
static	long	_qty;	// Number of samples to send per block.



/******************************************************************************
*
*	Function:	_output_buffer_status
*
*	Purpose:
*
*		Get the status of the output buffer.
*
*	Arguments:
*
*		fd			The handle for the board to access.
*
*		empty		Indicate here if the buffer is empty.
*
*	Returned:
*
*		>= 0		The number of errors seen.
*
******************************************************************************/

static int _output_buffer_status(int fd, int* empty)
{
	int	errs	= 0;
	s32	lvl;
	int	ret;
	s32	sts;

	lvl			= 0;
	ret			= aio_ioctl(fd, AIO_IOCTL_AO_BUF_THR_LVL, &lvl);
	errs		+= ret ? 1 : 0;

	ret			= aio_ioctl(fd, AIO_IOCTL_AO_BUF_THR_STS, &sts);
	errs		+= ret ? 1 : 0;
	empty[0]	= (sts == AIO_AO_BUF_THR_STS_CLEAR) ? 1 : 0;
	return(errs);
}



/******************************************************************************
*
*	Function:	_io_write
*
*	Purpose:
*
*		Write the data to the device.
*
*	Arguments:
*
*		fd		This is the handle used to access the device.
*
*		chans	The number of channels being used.
*
*		errs	This is the number of errors seen so far. We don't do the
*				work if errors have been see.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
******************************************************************************/

static int _io_write(int fd, int chans, int errs)
{
	struct timeval	begin;
	int				empty;
	struct timeval	end;
	long			loop;
	char*			ptr;
	int				send;
	int				sent	= 0;
	long long		sps;
	long			tmp;
	long long		total	= 0;
	long			us;
	u32				zero[4]	= { 0, 0, 0, 0 };

	gsc_label("Writing Data");

	for (;;)	// A convenience loop.
	{
		if (errs)
		{
			errs	= 0;
			printf("SKIPPED   (due to errors)\n");
			break;
		}

		errs	= 0;

		// Write the data to the device.
		send	= chans * 4;
		tmp		= aio_write(fd, zero, send);

		if (tmp < 0)
			errs	= 1;

		gettimeofday(&begin, NULL);

		for (loop = 0; loop < _loops; loop++)
		{
			send	= _qty * 4;
			ptr		= (char*) _data;

			for (;;)
			{
				sent	= aio_write(fd, ptr, send);

				if (sent < 0)
					break;

				total	+= sent;

				if (sent == send)
					break;

				ptr		+= sent;
				send	-= sent;
			}

			if (sent < 0)
				break;
		}

		gettimeofday(&end, NULL);

		send	= chans * 4;
		tmp		= aio_write(fd, zero, send);

		if (tmp < 0)
			errs	= 1;

		// Wait for the output buffer to empty.

		for (;;)
		{
			errs	= _output_buffer_status(fd, &empty);

			if ((errs) || (empty))
				break;
		}

		// Compute the results.

		if (sent < 0)
		{
			errs	= 1;
			printf("FAIL <---  (aio_write error)\n");
		}
		else
		{
			us	= (end.tv_sec - begin.tv_sec) * 1000000L
				+ (end.tv_usec - begin.tv_usec);
			sps	= (total * 10000000L / 4 / us + 5) / 10;

			printf(	"PASS  (");
			gsc_label_long_comma(total);
			printf(" Bytes, %ld.%06ld Seconds, ", us / 1000000, us % 1000000);
			gsc_label_long_comma(sps);
			printf(" S/S)\n");
		}

		break;
	}

	return(errs);
}



/******************************************************************************
*
*	Function:	_init_data
*
*	Purpose:
*
*		initialize our transmit data buffer.
*
*	Arguments:
*
*		fd			The handle for the board to access.
*
*		chans	The number of channels we're using.
*
*		rate	The data output clock rate.
*
*		seconds	Generate data for this period of time.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
******************************************************************************/

static int _init_data(int fd, int chans, long rate, int seconds)
{
	#define PIE		(3.1415962)

	u32*		dest;
	int			errs	= 0;
	int			i;
	u32			max;
	long		period_len;	// The length of the pattern in samples.
	long		period_tot;	// Pattern samples total for all channels.
	long		period;		// Pattern length in x units.
	s32			res;
	int			samp;
	float		samp_dat;	// Total sample size of the data buffer.
	float		samp_sec;	// Samples Per Second for all active channels
	float		samp_tot;	// Samples Total for all active channels
	const char*	units;

	gsc_label("Initializing Data");
	errs	= aio_query(fd, -1, 0, AIO_QUERY_RES_BITS, &res);
	max		= (0x1UL << res) - 1;
	memset(_data, 0, sizeof(_data));
	samp_sec	= ((float) rate) * chans;
	samp_tot	= ((float) rate) * chans * seconds;
	samp_dat	= sizeof(_data) / 4;

	period_len	= (rate <= 100) ? rate				//      10-      100 ->    10-   100
				: (rate <= 1000) ? (rate / 2)		//     101-    1,000 ->    50-   500
				: (rate <= 10000) ? (rate / 5)		//   1,001-   10,000 ->   200- 2,000
				: (rate <= 100000) ? (rate / 10)	//  10,001-  100,000 -> 1,000-10,000
				: (rate / 100);						// 100,001-1,000,000 -> 1,000-10,000
	period_tot	= period_len * chans;
	period		= (long) (1000000000. / rate * period_len);
	units		= "ns";

	if ((period % 1000) == 0)
	{
		period	/= 1000;
		units	= "us";
	}

	if ((period % 1000) == 0)
	{
		period	/= 1000;
		units	= "ms";
	}

	if (samp_tot <= SIZEOF_ARRAY(_data))
	{
		// We can get the entire image in the buffer.
		_loops	= 1;
		_qty	= (long) samp_tot;
	}
	else if (samp_sec <= SIZEOF_ARRAY(_data))
	{
		// We can get one second of the image in the buffer.
		_loops	= seconds;
		_qty	= (long) samp_sec;
	}
	else if (period_tot <= SIZEOF_ARRAY(_data))
	{
		// We can get one period of the image in the buffer.
		_loops	= (long) (seconds * (samp_sec / period_tot));
		_qty	= period_tot;
	}
	else
	{
		// We can get only a fraction of a period of the image in the buffer.
		_qty	= (long) ((samp_dat / chans) * chans);
		_loops	= (long) ((samp_tot + _qty - 1) / _qty);
	}

	// Initialize the data buffer.
	// The data is encoded in OFFSET BINARY format.
	dest	= _data;;

	for (i = 0, samp = 0; i < _qty; i += chans, samp++)
	{
		samp	= (samp < period_len) ? samp : 0;

		// 1st channel: sawtooth wave
		*dest++	= ((u16) (max - (max * samp / (period_len - 1))))
				| 0x00000L;	// channel tag

		// 2nd channel: ramp wave
		*dest++	= ((u16) (max * samp / (period_len - 1)))
				| 0x10000L;	// channel tag

		// 3rd channel: square wave
		*dest++	= ((samp < (period_len / 2)) ? max : 0)
				| 0x20000L;	// channel tag

		// 4th channel: sine wave
		*dest++	= (u32) (((long) (max >> 1)) * (1 + sin((2 * PIE / period_len) * samp)))
				| 0x70000L;	// channel tag and group end flag
	}

	if (errs == 0)
		printf("DONE   (pattern length: %ld samples, %ld %s)\n", period_len, period, units);

	return(errs);
}



/******************************************************************************
*
*	Function:	aio_out
*
*	Purpose:
*
*		Ganerate Analog Output.
*
*	Arguments:
*
*		fd		The file descriptor for the board to access.
*
*		rate	The desirted data output rate.
*
*		seconds	Generate data for this period of time.
*
*		io_mode	The I/O mode to use for reading (PIO, DMA, DMDMA).
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
******************************************************************************/

int aio_out(int fd, long rate, int seconds, int io_mode)
{
	int	chans	= 4;
	int	errs	= 0;

	gsc_label("Analog Output");
	printf("\n");
	gsc_label_level_inc();

	errs	+= _init_data(fd, chans, rate, seconds);
	errs	+= aio_config_ao(fd, -1, 1, rate);
	errs	+= aio_rx_io_mode(fd, -1, 1, io_mode, NULL);
	errs	+= _io_write(fd, chans, errs);

	gsc_label_level_dec();
	return(errs);
}



