// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AIO/16AIO_Linux_4.x.x.x_DN/samples/din/dio.c $
// $Rev: 43223 $
// $Date: 2018-07-08 17:48:15 -0500 (Sun, 08 Jul 2018) $

// 16AIO: Sample Application: source file

#include "main.h"



/******************************************************************************
*
*	Function:	_bin_show
*
*	Purpose:
*
*		Display the given data in binary.
*
*	Arguments:
*
*		data	The data to show.
*
*	Returned:
*
*		None.
*
******************************************************************************/

static void _bin_show(u16 data)
{
	u16	bit		= 0x8000;
	int	i;
	int	nibble;

	for (nibble = 3; nibble >= 0; nibble--)
	{
		for (i = 0; i <= 3; i++, bit /= 2)
		{
			if (bit & data)
				printf("1");
			else
				printf("0");
		}

		printf(" ");
	}
}



/******************************************************************************
*
*	Function:	_dev_dio_in
*
*	Purpose:
*
*		Read from the GPIO input.
*
*	Arguments:
*
*		fd		The file descriptor for the board to access.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
******************************************************************************/

static int _dev_dio_in(int fd)
{
	u16	data;
	int	errs	= 0;
	int	ret;
	u32	val;

	ret		= aio_ioctl(fd, AIO_IOCTL_DIO_READ, &val);
	errs	= ret ? 1 : 0;
	data	= (u16) (0xFFFF & val);
	_bin_show(data);
	return(errs);
}



/******************************************************************************
*
*	Function:	dio_in
*
*	Purpose:
*
*		Perform a GPIO input test.
*
*	Arguments:
*
*		fd		The file descriptor for the board to access.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
******************************************************************************/

int dio_in(int fd)
{
	int	errs	= 0;
	int	i;

	for (;;)
	{
		errs	+= aio_initialize(fd, -1, 1);
		errs	+= aio_dio_dir_out(fd, -1, 1, 0, NULL);

		if (errs)
			break;

		printf("    DIO In\n");
		printf("    FEDC BA98 7654 3210\n");
		printf("    ==== ==== ==== ====\n");

		for (i = 0; i < 16; i++)
		{
			printf("    ");
			errs	+= _dev_dio_in(fd);
			printf("\n");
			gsc_time_sleep_ms(250);
		}

		gsc_label_level_inc();
		errs	+= aio_initialize(fd, -1, 0);
		gsc_label("Status");
		printf("%s\n", errs ? "FAIL <---" : "PASS");
		gsc_label_level_dec();
		break;
	}

	return(errs);
}



