// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AIO/16AIO_Linux_4.x.x.x_DN/samples/dout/dio.c $
// $Rev: 43224 $
// $Date: 2018-07-08 17:48:29 -0500 (Sun, 08 Jul 2018) $

// 16AIO: Sample Application: source file

#include "main.h"



/******************************************************************************
*
*	Function:	_bin_show
*
*	Purpose:
*
*		Display the given data in binary.
*
*	Arguments:
*
*		data	The data to show.
*
*	Returned:
*
*		None.
*
******************************************************************************/

static void _bin_show(u16 data)
{
	u16	bit		= 0x8000;
	int	i;
	int	nibble;

	for (nibble = 3; nibble >= 0; nibble--)
	{
		for (i = 0; i <= 3; i++, bit /= 2)
		{
			if (bit & data)
				printf("1");
			else
				printf("0");
		}

		printf(" ");
	}
}



/******************************************************************************
*
*	Function:	_dev_dio_out
*
*	Purpose:
*
*		Write to the GPIO output.
*
*	Arguments:
*
*		fd		The file descriptor for the board to access.
*
*		data	The data to write.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
******************************************************************************/

static int _dev_dio_out(int fd, u16 data)
{
	int	errs;
	int	ret;
	u32	val		= data;

	_bin_show(data);
	ret		= aio_ioctl(fd, AIO_IOCTL_DIO_WRITE, &val);
	errs	= ret ? 1 : 0;
	return(errs);
}



/******************************************************************************
*
*	Function:	_dev_dio_out_16
*
*	Purpose:
*
*		Write to the 16 GPIO output lines.
*
*	Arguments:
*
*		fd		The file descriptor for the board to access.
*
*		data	The data to write.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
******************************************************************************/

static int _dev_dio_out_16(int fd, u16 data)
{
	int	errs	= 0;

	printf("    ");
	errs	+= _dev_dio_out(fd, data);
	printf("\n");
	gsc_time_sleep_ms(250);
	return(errs);
}



/******************************************************************************
*
*	Function:	dio_out
*
*	Purpose:
*
*		Perform a GPIO output test.
*
*	Arguments:
*
*		fd		The file descriptor for the board to access.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
******************************************************************************/

int dio_out(int fd)
{
	int	errs	= 0;
	int	i;

	for (;;)	// A convenience loop
	{
		errs	+= aio_initialize(fd, -1, 1);
		errs	+= aio_dio_dir_out(fd, -1, 1, 0x3, NULL);

		if (errs)
			break;

		printf("    DIO Out\n");
		printf("    FEDC BA98 7654 3210\n");
		printf("    ==== ==== ==== ====\n");

		errs	+= _dev_dio_out_16(fd, 0x0000);
		errs	+= _dev_dio_out_16(fd, 0xFFFF);
		errs	+= _dev_dio_out_16(fd, 0x0000);
		errs	+= _dev_dio_out_16(fd, 0xFFFF);
		errs	+= _dev_dio_out_16(fd, 0x0000);

		for (i = 0; i <= 15; i++)
			errs	+= _dev_dio_out_16(fd, 0x1 << i);

		errs	+= _dev_dio_out_16(fd, 0x0000);

		for (i = 15; i >= 0; i--)
			errs	+= _dev_dio_out_16(fd, 0x1 << i);

		errs	+= _dev_dio_out_16(fd, 0x0000);
		errs	+= _dev_dio_out_16(fd, 0xFFFF);
		errs	+= _dev_dio_out_16(fd, 0x0000);
		errs	+= _dev_dio_out_16(fd, 0xFFFF);
		errs	+= _dev_dio_out_16(fd, 0x0000);

		gsc_label_level_inc();
		errs	+= aio_initialize(fd, -1, 0);
		gsc_label("Status");
		printf("%s\n", errs ? "FAIL <---" : "PASS");
		gsc_label_level_dec();
		break;
	}

	return(errs);
}



