// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AIO/16AIO_Linux_4.x.x.x_DN/samples/id/id.c $
// $Rev: 43225 $
// $Date: 2018-07-08 17:48:41 -0500 (Sun, 08 Jul 2018) $

// 16AIO: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _input_modes(void)
{
	gsc_label("Input Modes");
	printf("Differential, Single-Ended, Zero Test, +Vref Test, Chan 0/1/2/3 Loopback\n");
	return(0);
}



//*****************************************************************************
static int _voltage_ranges(void)
{
	gsc_label("Voltage Ranges");
	printf("+-10V, +-5V, +-2.5V\n");
	return(0);
}



//*****************************************************************************
static int _channel_qty(int fd)
{
	s32	bits;
	int	errs	= 0;
	int	ret;

	gsc_label("Input Channels");
	ret		= aio_query(fd, -1, 0, AIO_QUERY_RES_BITS, &bits);
	errs	+= ret;

	if (ret == 0)
	{
		printf(	"16 Differential or 32 Single Ended  (%ld-bit resolution)\n",
				(long) bits);
	}

	gsc_label("Output Channels");
	ret		= aio_query(fd, -1, 0, AIO_QUERY_RES_BITS, &bits);
	errs	+= ret;

	if (ret == 0)
		printf("4  (%ld-bit resolution)\n", (long) bits);

	return(errs);
}



//*****************************************************************************
static int _master_clock(int fd)
{
	int	errs	= 0;
	s32	freq;

	gsc_label("Master Clock");

	errs	+= aio_query(fd, -1, 0, AIO_QUERY_FREF_DEFAULT, &freq);

	if (errs)
	{
	}
	else if (freq <= 0)
	{
		errs	= 1;
		printf(	"FAIL <---  (%d. frequency response error: %ld)\n",
				__LINE__,
				(long) freq);
	}
	else
	{
		gsc_label_long_comma(freq);
		printf(" Hz  (default)\n");
	}

	return(errs);
}



//*****************************************************************************
static int _rate_generators(void)
{
	gsc_label("Rate Generators");
	printf("2  (A-B)\n");
	return(0);
}



//*****************************************************************************
static int _clocking_sources(void)
{
	gsc_label("Clocking Sources");
	printf("Rate Generator(s), External, Software\n");
	return(0);
}



//*****************************************************************************
static int _nrate_range(int fd)
{
	int	errs	= 0;
	s32	mask;
	s32	max;
	s32	min;

	gsc_label("AI Nrate Range");
	errs	+= aio_query(fd, -1, 0, AIO_QUERY_NRATE_AI_MASK, &mask);
	errs	+= aio_query(fd, -1, 0, AIO_QUERY_NRATE_AI_MAX, &max);
	errs	+= aio_query(fd, -1, 0, AIO_QUERY_NRATE_AI_MIN, &min);

	if (errs == 0)
	{
		gsc_label_long_comma(min);
		printf(" to ");
		gsc_label_long_comma(max);
		printf("  (mask 0x%lX)\n", (long) mask);
	}

	gsc_label("AO Nrate Range");
	errs	+= aio_query(fd, -1, 0, AIO_QUERY_NRATE_AO_MASK, &mask);
	errs	+= aio_query(fd, -1, 0, AIO_QUERY_NRATE_AO_MAX, &max);
	errs	+= aio_query(fd, -1, 0, AIO_QUERY_NRATE_AO_MIN, &min);

	if (errs == 0)
	{
		gsc_label_long_comma(min);
		printf(" to ");
		gsc_label_long_comma(max);
		printf("  (mask 0x%lX)\n", (long) mask);
	}

	return(errs);
}



//*****************************************************************************
static int _fgen_range(int fd)
{
	int	errs	= 0;
	s32	max;
	s32	min;

	gsc_label("AI Fgen Range");
	errs	+= aio_query(fd, -1, 0, AIO_QUERY_FGEN_AI_MAX, &max);
	errs	+= aio_query(fd, -1, 0, AIO_QUERY_FGEN_AI_MIN, &min);

	if (errs == 0)
	{
		gsc_label_long_comma(min);
		printf(" to ");
		gsc_label_long_comma(max);
		printf(" Hz\n");
	}

	gsc_label("AO Fgen Range");
	errs	+= aio_query(fd, -1, 0, AIO_QUERY_FGEN_AO_MAX, &max);
	errs	+= aio_query(fd, -1, 0, AIO_QUERY_FGEN_AO_MIN, &min);

	if (errs == 0)
	{
		gsc_label_long_comma(min);
		printf(" to ");
		gsc_label_long_comma(max);
		printf(" Hz\n");
	}

	return(errs);
}



//*****************************************************************************
static int _fsamp_range(int fd)
{
	int	errs	= 0;
	s32	max;			// Sample Rate Maximum
	s32	min;			// Sample Rate Minimum

	gsc_label("AI Sample Rates");
	errs	+= aio_query(fd, -1, 0, AIO_QUERY_FSAMP_AI_MAX, &max);
	errs	+= aio_query(fd, -1, 0, AIO_QUERY_FSAMP_AI_MIN, &min);

	if (errs == 0)
	{
		gsc_label_long_comma(min);
		printf(" to ");
		gsc_label_long_comma(max);
		printf(" S/S\n");
	}

	gsc_label("AO Sample Rates");
	errs	+= aio_query(fd, -1, 0, AIO_QUERY_FSAMP_AO_MAX, &max);
	errs	+= aio_query(fd, -1, 0, AIO_QUERY_FSAMP_AO_MIN, &min);

	if (errs == 0)
	{
		gsc_label_long_comma(min);
		printf(" to ");
		gsc_label_long_comma(max);
		printf(" S/S\n");
	}

	return(errs);
}



//*****************************************************************************
static int _sampling_modes(void)
{
	gsc_label("Input Modes");
	printf("Continuous, Burst\n");

	gsc_label("Output Modes");
	printf("Immediate, Synchronous, Burst, Looping\n");
	return(0);
}



//*****************************************************************************
static int _fifo_size(int fd)
{
	int	errs	= 0;
	s32	size;

	gsc_label("Rx FIFO Size");
	errs	+= aio_query(fd, -1, 0, AIO_QUERY_FIFO_SIZE_RX, &size);
	gsc_label_long_comma(size);
	printf(" Samples Deep");
	printf("  (%ldK)\n", (long) size / 1024);

	gsc_label("Tx FIFO Size");
	errs	+= aio_query(fd, -1, 0, AIO_QUERY_FIFO_SIZE_TX, &size);
	gsc_label_long_comma(size);
	printf(" Samples Deep");
	printf("  (%ldK)\n", (long) size / 1024);

	return(errs);
}



//*****************************************************************************
static int _digital_io(void)
{
	gsc_label("Digital I/O");
	printf("16 Bi-Directional Signals  (2 ports, 8-bits each)\n");
	return(0);
}



//*****************************************************************************
static int _initialize(int fd)
{
	int	errs	= 0;
	s32	ms;

	gsc_label("Initialize Period");

	errs	+= aio_query(fd, -1, 0, AIO_QUERY_INIT_MS, &ms);

	if (errs)
	{
	}
	else if (ms <= 0)
	{
		errs	= 1;
		printf(	"FAIL <---  (%d. ms response error: %ld)\n",
				__LINE__,
				(long) ms);
	}
	else
	{
		gsc_label_long_comma(ms);
		printf(" ms maximum\n");
	}

	return(errs);
}



//*****************************************************************************
static int _auto_calibrate(int fd)
{
	int	errs	= 0;
	s32	ms;

	gsc_label("Auto-Calibrate Period");

	errs	+= aio_query(fd, -1, 0, AIO_QUERY_AUTO_CAL_MS, &ms);

	if (errs)
	{
	}
	else if (ms <= 0)
	{
		errs	= 1;
		printf(	"FAIL <---  (%d. ms response error: %ld)\n",
				__LINE__,
				(long) ms);
	}
	else
	{
		gsc_label_long_comma(ms);
		printf(" ms maximum\n");
	}

	return(errs);
}



//*****************************************************************************
static int _register_map(int fd)
{
	int	errs	= 0;

	printf("\n");
	errs	= aio_reg_list(fd, 0);
	return(errs);
}



/******************************************************************************
*
*	Function:	id_device
*
*		Identify the device and its features.
*
*	Arguments:
*
*		fd		The handle to the board to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int id_device(int fd)
{
	int	errs	= 0;

	gsc_label("Board Features");
	printf("\n");

	gsc_label_level_inc();

	errs	+= _input_modes();
	errs	+= _voltage_ranges();
	errs	+= _channel_qty(fd);
	errs	+= _rate_generators();
	errs	+= _master_clock(fd);
	errs	+= _clocking_sources();
	errs	+= _nrate_range(fd);
	errs	+= _fgen_range(fd);
	errs	+= _fsamp_range(fd);
	errs	+= _sampling_modes();
	errs	+= _fifo_size(fd);
	errs	+= _digital_io();
	errs	+= _initialize(fd);
	errs	+= _auto_calibrate(fd);

	gsc_label_level_dec();

	errs	+= _register_map(fd);

	return(errs);
}


