// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AIO/16AIO_Linux_4.x.x.x_DN/samples/savedata/savedata.c $
// $Rev: 43230 $
// $Date: 2018-07-08 17:49:49 -0500 (Sun, 08 Jul 2018) $

// 16AIO: Sample Application: source file

#include "main.h"



// #defines *******************************************************************

#define _1M		(1024L * 1024L)



// variables ******************************************************************

static	u32	_buffer[_1M];



/******************************************************************************
*
*	Function:	_channels
*
*	Purpose:
*
*		Check to see how many channels the board has.
*
*	Arguments:
*
*		fd		The handle for the board to access.
*
*		chan	Report the number of channels here.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

static int _channels(int fd)
{
	int	errs	= 0;
	s32	get		= -1;
	s32	num		= -1;
	int	ret;

	gsc_label("Input Channels");

	get		= -1;
	ret		= aio_ioctl(fd, AIO_IOCTL_AI_SCAN_SIZE, &get);
	errs	+= ret ? 1 : 0;
	ret		= aio_ioctl(fd, AIO_IOCTL_AI_SCAN_1_CHAN, &num);
	errs	+= ret ? 1 : 0;

	switch (get)
	{
		default:

			errs++;
			printf(	"FAIL <---  (%d. unrecognixed option: %ld)\n",
					__LINE__,
					(long) get);
			break;

		case AIO_AI_SCAN_SIZE_SINGLE:

			printf("1 Channel (#%ld)\n", (long) num);
			break;

		case AIO_AI_SCAN_SIZE_0_1:

			printf("2 Channel (#0 to #1)\n");
			break;

		case AIO_AI_SCAN_SIZE_0_3:

			printf("4 Channel (#0 to #3)\n");
			break;

		case AIO_AI_SCAN_SIZE_0_7:

			printf("8 Channel (#0 to #7)\n");
			break;

		case AIO_AI_SCAN_SIZE_0_15:

			printf("16 Channel (#0 to #15)\n");
			break;

		case AIO_AI_SCAN_SIZE_0_31:

			printf("32 Channel (#0 to #21)\n");
			break;
	}

	return(errs);
}



/******************************************************************************
*
*	Function:	_read_data
*
*	Purpose:
*
*		Read data into the buffer.
*
*	Arguments:
*
*		fd		The handle for the board to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

static int _read_data(int fd)
{
	int		errs	= 0;
	long	get		= sizeof(_buffer);
	int		got;

	gsc_label("Reading");
	got	= aio_read(fd, _buffer, get);

	if (got < 0)
	{
		errs	= 1;
		printf(	"FAIL <---  (error %d)\n", got);
	}
	else if (got != get)
	{
		errs	= 1;
		printf(	"FAIL <---  (got %ld samples, requested %ld)\n",
				(long) got,
				(long) get);
	}
	else
	{
		errs	= 0;
		printf(	"PASS  (%ld samples)\n",
				(long) get);
	}

	return(errs);
}



/******************************************************************************
*
*	Function:	_save_data
*
*	Purpose:
*
*		Save the read data to a text file.
*
*	Arguments:
*
*		errs	have there been any errors so far?
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

static int _save_data(int errs)
{
	FILE*		file;
	int			i;
	long		l;
	const char*	name	= "data.txt";
	long		samples	= sizeof(_buffer) / 4;

	gsc_label("Saving");

	for (;;)
	{
		if (errs)
		{
			printf("SKIPPED  (errors)\n");
			errs	= 0;
			break;
		}

		file	= fopen(name, "w+b");

		if (file == NULL)
		{
			printf("FAIL <---  (unable to create %s)\n", name);
			errs	= 1;
			break;
		}

		for (l = 0; l < samples; l++)
		{
			if ((l) && (_buffer[l] & 0x10000))
			{
				i	= fprintf(file, "\r\n");

				if (i != 2)
				{
					printf("FAIL <---  (fprintf() failure to %s)\n", name);;
					errs	= 1;
					break;
				}
			}

			i	= fprintf(file, "  %08lX", (long) _buffer[l]);

			if (i != 10)
			{
				printf("FAIL <---  (fprintf() failure to %s)\n", name);
				errs	= 1;
				break;
			}
		}

		fclose(file);

		if (errs == 0)
			printf("PASS  (%s)\n", name);

		break;
	}

	return(errs);
}



//*****************************************************************************
int save_data(int fd, s32 io_mode)
{
	int		errs	= 0;

	gsc_label("Capture & Save");
	printf("\n");
	gsc_label_level_inc();

	errs	+= aio_config_ai(fd, -1, 1, 15000);
	errs	+= aio_rx_io_mode(fd, -1, 1, io_mode, NULL);
	errs	+= _channels(fd);
	errs	+= aio_ai_buf_clear(fd, -1, 1);
	errs	+= _read_data(fd);
	errs	+= _save_data(errs);

	gsc_label_level_dec();
	return(errs);
}



