// $URL: http://subversion:8080/svn/gsc/trunk/drivers/gsc_common/utils/intime/os_util_thread.c $
// $Rev: 43207 $
// $Date: 2018-07-08 17:04:20 -0500 (Sun, 08 Jul 2018) $

// INtime: Utility: source file

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "gsc_utils.h"



// #defines *******************************************************************

// TNtime doesn't auto-grow the stack.
// This is a good size, but may be too small in many cases.
#define STACK_SIZE			(64 * 1024)



// data types *****************************************************************

typedef struct
{
	int		(*function)(void* arg);
	void*	arg;
} _thread_data_t;



//*****************************************************************************
static void _thread_func(void* arg)
{
	_thread_data_t*	data	= (_thread_data_t*) arg;
	_thread_data_t	tmp		= data[0];

	free(arg);
	(tmp.function)(tmp.arg);
}



//*****************************************************************************
int os_thread_create(	os_thread_t*	thread,
						const char*		name,
						int				(*function)(void* arg),
						void*			arg)
{
	_thread_data_t*	data;
	int				errs	= 0;
	WORD			priority;
	RTHANDLE		process;

	for (;;)	// A convenience loop.
	{
		if ((thread == NULL) || (name == NULL) || (function == NULL))
		{
			printf("FAIL <---  (invalid arguments)\n");
			errs++;
			break;
		}

		memset(thread, 0, sizeof(os_thread_t));
		strncpy(thread->name, name, sizeof(thread->name));
		thread->name[sizeof(thread->name) - 1]	= 0;
		data	= malloc(sizeof(_thread_data_t));

		if (data == NULL)
		{
			printf("FAIL <---  (malloc failed)\n");
			errs++;
			break;
		}

		data->function	= function;
		data->arg		= arg;
		process			= GetRtThreadHandles(THIS_PROCESS);
		priority		= GetRtThreadPriority(process);
		thread->handle	= CreateRtThread(	priority - 1,
											_thread_func,
											STACK_SIZE,
											(void*) data);


		if (thread->handle == BAD_RTHANDLE)
		{
			printf("FAIL <---  (thread creation failed)\n");
			free(data);
			errs++;
			break;
		}

		break;
	}

	return(errs);
}



//*****************************************************************************
int os_thread_destroy(os_thread_t* thread)
{
	int	errs	= 0;

	if (thread)
	{
		// We do not explicitly delete threads from outside the thread. This
		// is to prevent that thread from leaking resources. Instead, each
		// thread does this itself. For additional information refer to the
		// documentation for DeleteRtThread().
		memset(thread, 0, sizeof(os_thread_t));
	}
	else
	{
		printf("FAIL <---  (invalid arguments)\n");
		errs++;
	}

	return(errs);
}


