// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AIO/16AIO_Linux_4.x.x.x_DN/utils/util_config_ai.c $
// $Rev: 43213 $
// $Date: 2018-07-08 17:38:25 -0500 (Sun, 08 Jul 2018) $

// 16AIO: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	aio_config_ai
*
*	Purpose:
*
*		Configure the given board's analog input options.
*
*	Arguments:
*
*		fd		The handle to use to access the driver.
*
*		index	The index of the board to access. Ignore if < 0.
*
*		verbose	Work verbosely?
*
*		fsamp	This is the desired Fsamp rate, of -1 to use the default.
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

int aio_config_ai(int fd, int index, int verbose, s32 fsamp)
{
	int	errs	= 0;
	s32	nrate	= 0xFFFF;

	if (verbose)
	{
		gsc_label_index("Input Configuration", index);
		gsc_label_level_inc();
		printf("\n");
	}

	errs	+= aio_initialize			(fd, index, verbose);
	errs	+= aio_query				(fd, index, verbose, AIO_QUERY_DEVICE_TYPE,		NULL);
	errs	+= aio_rx_io_mode			(fd, index, verbose, GSC_IO_MODE_PIO,			NULL);
	errs	+= aio_rx_io_timeout		(fd, index, verbose, 30,						NULL);
	errs	+= aio_irq0_sel				(fd, index, verbose, AIO_IRQ0_IDLE_INIT,		NULL);
	errs	+= aio_irq1_sel				(fd, index, verbose, AIO_IRQ1_IDLE,				NULL);
	errs	+= aio_irq2_sel				(fd, index, verbose, AIO_IRQ2_IDLE,				NULL);
	errs	+= aio_data_format			(fd, index, verbose, AIO_DATA_FORMAT_OFF_BIN,	NULL);
	errs	+= aio_ai_mode				(fd, index, verbose, AIO_AI_MODE_DIFF,			NULL);
	errs	+= aio_range				(fd, index, verbose, AIO_RANGE_10V,				NULL);
	errs	+= aio_ai_buf_thr_lvl		(fd, index, verbose, 16L * 1024,				NULL);
	errs	+= aio_ai_scan_clk_src		(fd, index, verbose, AIO_AI_SCAN_CLK_SRC_RGA,	NULL);
	errs	+= aio_ai_scan_size			(fd, index, verbose, AIO_AI_SCAN_SIZE_0_15,		NULL);
	errs	+= aio_fsamp_ai_compute		(fd, index, verbose, fsamp, &nrate,				NULL);

	errs	+= aio_rga_nrate			(fd, index, verbose, nrate,						NULL);
	errs	+= aio_rga_enable			(fd, index, verbose, AIO_GEN_ENABLE_YES,		NULL);
	errs	+= aio_rgb_enable			(fd, index, verbose, AIO_GEN_ENABLE_NO,			NULL);
	errs	+= aio_fsamp_ai_report_all	(fd, index, verbose,							NULL);

//	errs	+= aio_auto_calibrate		(fd, index, verbose);	// Cable may need disconnecting for this.
	errs	+= aio_ai_buf_clear			(fd, index, verbose);

	if (verbose)
		gsc_label_level_dec();

	return(errs);
}


