// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AIO/16AIO_Linux_4.x.x.x_DN/utils/util_irq1_sel.c $
// $Rev: 43213 $
// $Date: 2018-07-08 17:38:25 -0500 (Sun, 08 Jul 2018) $

// 16AIO: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	aio_irq1_sel
*
*	Purpose:
*
*		Provide a visual wrapper for the AIO_IOCTL_IRQ1_SEL service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		This is the value to apply.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int aio_irq1_sel(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[128];
	int			errs;
	const char*	ptr;
	int			ret;

	if (verbose)
		gsc_label_index("IRQ 1 Select", index);

	ret		= aio_ioctl(fd, AIO_IOCTL_IRQ1_SEL, &set);
	errs	= ret ? 1 : 0;


	switch (set)
	{
		default:

			errs++;
			ptr	= buf;
			sprintf(buf, "Unrecognized option: 0x%lX", (long) set);
			break;

		case AIO_IRQ1_IDLE:

			ptr	= "Idle/Disable";
			break;

		case AIO_IRQ1_AI_BUF_THR_L2H:

			ptr	= "AI Buffer Threshold Low-to-High";
			break;

		case AIO_IRQ1_AI_BUF_THR_H2L:

			ptr	= "AI Buffer Threshold High-to-Low";
			break;
	}

	if (verbose)
		printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", ptr);

	if (get)
		get[0]	= set;

	return(errs);
}


