// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AIO/16AIO_Linux_4.x.x.x_DN/utils/util_query.c $
// $Rev: 43215 $
// $Date: 2018-07-08 17:42:32 -0500 (Sun, 08 Jul 2018) $

// 16AIO: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	aio_query
*
*	Purpose:
*
*		Provide a visual wrapper for the AIO_IOCTL_QUERY service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		This is the query option to access.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int aio_query(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[128];
	int			errs	= 0;
	const char*	ptr;
	s32			query	= set;
	int			ret;
	s32			tmp;

	if (verbose)
	{
		switch (query)
		{
			default:

				errs++;
				ptr	= buf;
				printf(buf, "Query Error (#%ld)", (long) query);
				break;

			case AIO_QUERY_AUTO_CAL_MS:		ptr	= "Auto-Cal Period";	break;
			case AIO_QUERY_COUNT:			ptr	= "Query Options";		break;
			case AIO_QUERY_DEVICE_TYPE:		ptr	= "Device Type";		break;
			case AIO_QUERY_FGEN_AI_MAX:		ptr	= "A/I Fgen Max";		break;
			case AIO_QUERY_FGEN_AI_MIN:		ptr	= "A/I Fgen Min";		break;
			case AIO_QUERY_FGEN_AO_MAX:		ptr	= "A/O Fgen Max";		break;
			case AIO_QUERY_FGEN_AO_MIN:		ptr	= "A/O Fgen Min";		break;
			case AIO_QUERY_FIFO_SIZE_RX:	ptr	= "Rx FIFO Size";		break;
			case AIO_QUERY_FIFO_SIZE_TX:	ptr	= "Tx FIFO Size";		break;
			case AIO_QUERY_FREF_DEFAULT:	ptr	= "Master Clock";		break;
			case AIO_QUERY_FSAMP_AI_MAX:	ptr	= "A/I Fsamp Max";		break;
			case AIO_QUERY_FSAMP_AI_MIN:	ptr	= "A/I Fsamp Min";		break;
			case AIO_QUERY_FSAMP_AO_MAX:	ptr	= "A/O Fsamp Max";		break;
			case AIO_QUERY_FSAMP_AO_MIN:	ptr	= "A/O Fsamp Min";		break;
			case AIO_QUERY_INIT_MS:			ptr	= "Initialize Period";	break;
			case AIO_QUERY_NRATE_AI_MASK:	ptr = "A/I Nrate Mask";		break;
			case AIO_QUERY_NRATE_AI_MAX:	ptr = "A/I Nrate Max";		break;
			case AIO_QUERY_NRATE_AI_MIN:	ptr = "A/I Nrate Min";		break;
			case AIO_QUERY_NRATE_AO_MASK:	ptr = "A/O Nrate Mask";		break;
			case AIO_QUERY_NRATE_AO_MAX:	ptr = "A/O Nrate Max";		break;
			case AIO_QUERY_NRATE_AO_MIN:	ptr = "A/O Nrate Min";		break;
			case AIO_QUERY_RES_BITS:		ptr	= "Resolution";			break;
		}

		gsc_label_index(ptr, index);
	}

	ret		= aio_ioctl(fd, AIO_IOCTL_QUERY, &set);
	errs	+= ret ? 1 : 0;

	if (verbose)
	{
		switch (query)
		{
			default:

				errs++;
				printf(buf, "%ld", (long) set);
				break;

			case AIO_QUERY_COUNT:
			case AIO_QUERY_NRATE_AI_MAX:
			case AIO_QUERY_NRATE_AI_MIN:
			case AIO_QUERY_NRATE_AO_MAX:
			case AIO_QUERY_NRATE_AO_MIN:

				sprintf(buf, "%ld", (long) set);
				break;

			case AIO_QUERY_NRATE_AI_MASK:
			case AIO_QUERY_NRATE_AO_MASK:

				sprintf(buf, "0x%lX", (long) set);
				break;

			case AIO_QUERY_RES_BITS:

				sprintf(buf, "%ld-bits", (long) set);
				break;

			case AIO_QUERY_FGEN_AI_MAX:
			case AIO_QUERY_FGEN_AI_MIN:
			case AIO_QUERY_FGEN_AO_MAX:
			case AIO_QUERY_FGEN_AO_MIN:
			case AIO_QUERY_FREF_DEFAULT:

				sprintf(buf, "%ld Hz", (long) set);
				break;

			case AIO_QUERY_AUTO_CAL_MS:
			case AIO_QUERY_INIT_MS:

				sprintf(buf, "%ld ms", (long) set);
				break;

			case AIO_QUERY_FSAMP_AI_MAX:
			case AIO_QUERY_FSAMP_AI_MIN:
			case AIO_QUERY_FSAMP_AO_MAX:
			case AIO_QUERY_FSAMP_AO_MIN:

				sprintf(buf, "%ld S/S", (long) set);
				break;

			case AIO_QUERY_FIFO_SIZE_RX:
			case AIO_QUERY_FIFO_SIZE_TX:

				ptr	= "";
				tmp	= set;

				if ((tmp) && ((tmp % 1024) == 0))
				{
					ptr	= "K";
					tmp	/= 1024;
				}

				sprintf(buf, "%ld%s Words", (long) tmp, ptr);
				break;

			case AIO_QUERY_DEVICE_TYPE:

				if (set == GSC_DEV_TYPE_16AIO)
				{
					strcpy(buf, "16AIO");
				}
				else
				{
					errs++;
					sprintf(buf, "INVALID: %ld", (long) set);
				}

				break;
		}

		if (errs)
			printf("FAIL <---  (%s)\n", buf);
		else
			printf("%s\n", buf);
	}

	if (get)
		get[0]	= set;

	return(errs);
}


